#include <config.h>
#ifndef lint
static const char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
#endif

#include <stdlib.h>
#include <string.h>

#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYPATCH 20070509

#define YYEMPTY (-1)
#define yyclearin    (yychar = YYEMPTY)
#define yyerrok      (yyerrflag = 0)
#define YYRECOVERING (yyerrflag != 0)

extern int yyparse(void);

static int yygrowstack(void);
#define YYPREFIX "yy"
#line 32 "p.y"
  
/*
 * DESCRIPTION
 *   Simple context-free grammar for parsing the control file. 
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Olivier Beyssac, <ob@r14.freenix.org>
 *  @author Kianusch Sayah Karadji <kianusch.sayah.karadji@sk-tech.net>
 *  @author Martin Pala <martinp@tildeslash.com>
 *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
 *  @author Rory Toma <rory@digeo.com>
 *  @version \$Id: p.y,v 1.300 2009/05/04 21:25:21 martinp Exp $
 */

#include <config.h>

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif 

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif 

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif 

#ifdef HAVE_GRP_H
#include <grp.h>
#endif 
  
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
  
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
  
#ifdef HAVE_TIME_H
#include <time.h>
#endif
  
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

#ifdef HAVE_ASM_PARAM_H
#include <asm/param.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif

#ifdef HAVE_SYSLOG_H
#include <syslog.h>
#endif

#ifndef HAVE_SOL_IP
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#endif

#ifdef HAVE_NETINET_IP_ICMP_H
#include <netinet/ip_icmp.h>
#endif

#ifdef HAVE_REGEX_H
#include <regex.h>
#endif
  
#include "net.h"
#include "monitor.h"
#include "protocol.h"
#include "engine.h"
#include "alert.h"
#include "process.h"
#include "ssl.h"
#include "device.h"


/* ------------------------------------------------------------- Definitions */

  struct IHavePrecedence {
    int daemon;
    int logfile;
    int pidfile;
  }; 

  struct myrate {
    unsigned count;
    unsigned cycles;
  }; 

  /* yacc interface */
  void  yyerror(const char *,...);
  void  yyerror2(const char *,...);
  void  yywarning(const char *,...);
  void  yywarning2(const char *,...);

  /* lexer interface */
  int yylex(void);
  extern FILE *yyin;
  extern int lineno;
  extern int arglineno;
  extern char *yytext;
  extern char *argyytext;
  extern char *currentfile;
  extern char *argcurrentfile;
  extern int buffer_stack_ptr;

  /* Local variables */
  static int cfg_errflag = FALSE;
  static Service_T tail = NULL;
  static Service_T current = NULL;
  static unsigned int eventset;
  static Request_T urlrequest = NULL;
  static Command_T command = NULL;
  static Command_T command1 = NULL;
  static Command_T command2 = NULL;
  static Service_T depend_list = NULL;
  static struct mygid gidset;
  static struct myuid uidset;
  static struct myperm permset;
  static struct mysize sizeset;
  static struct mymatch matchset;
  static struct myicmp icmpset;
  static struct mymail mailset;
  static struct myport portset;
  static struct mymailserver mailserverset;
  static struct myfilesystem filesystemset;
  static struct myresource resourceset;
  static struct mychecksum checksumset;
  static struct mytimestamp timestampset;
  static struct IHavePrecedence ihp = {FALSE, FALSE, FALSE};
  static struct myrate rate1 = {1, 1};
  static struct myrate rate2 = {1, 1};
  static char * htpasswd_file = NULL;
  static int    digesttype = DIGEST_CLEARTEXT;
  static int    hassystem = FALSE;

#define BITMAP_MAX (sizeof(long long) * 8)

  
/* -------------------------------------------------------------- Prototypes */

  static void  preparse();
  static void  postparse();
  static void  addservice(Service_T);
  static void  addmail(char *, Mail_T, Mail_T *, unsigned int, unsigned int);
  static void  createservice(int, char *, char *, int (*)(Service_T));
  static void  adddependant(char *);
  static void  addport(Port_T);
  static void  addresource(Resource_T);
  static void  addtimestamp(Timestamp_T, int);
  static void  addsize(Size_T);
  static void  addfilesystem(Filesystem_T);
  static void  addicmp(Icmp_T);
  static void *addprotocol(int);
  static void  addgeneric(Port_T, char*, char*);
  static void  addcommand(int, unsigned);
  static void  addargument(char *);
  static void  addmmonit(URL_T, int, int, char *);
  static void  addmailserver(MailServer_T);
  static int   addcredentials(char *, char *, int, int);
#ifdef HAVE_LIBPAM
  static void  addpamauth(char *, int);
#endif
  static void  addhtpasswdentry(char *, char *, int);
  static uid_t get_uid(char *, uid_t);
  static gid_t get_gid(char *, gid_t);
  static void  addchecksum(Checksum_T);
  static void  addperm(Perm_T);
  static void  addmatch(Match_T, int, int);
  static void  addmatchpath(Match_T, int);
  static void  adduid(Uid_T);
  static void  addgid(Gid_T);
  static void  addeuid(uid_t);
  static void  addegid(gid_t);
  static void  addeventaction(EventAction_T *, int, int);
  static void  seteventaction(EventAction_T *, int, int);
  static void  prepare_urlrequest(URL_T U);
  static void  seturlrequest(int, char *);
  static void  setlogfile(char *);
  static void  setpidfile(char *);
  static void  reset_mailset();
  static void  reset_mailserverset();
  static void  reset_portset();
  static void  reset_resourceset();
  static void  reset_timestampset();
  static void  reset_sizeset();
  static void  reset_checksumset();
  static void  reset_permset();
  static void  reset_uidset();
  static void  reset_gidset();
  static void  reset_filesystemset();
  static void  reset_icmpset();
  static void  reset_rateset();
  static void  check_name(char *);
  static void  check_timeout(int, int);
  static void  check_every(int);
  static int   check_perm(int);
  static void  check_hostname (char *);
  static void  check_exec(char *);
  static int   cleanup_hash_string(char *);
  static void  check_depend();
  static void  setsyslog(char *);
  static void describeAction(Action_T);
  static Command_T copycommand(Command_T);
  static int verifyMaxForward(int);  

#line 264 "p.y"
typedef union {
  URL_T url;
  float real;
  int   number;
  char *string;
} YYSTYPE;
#line 261 "y.tab.c"
#define IF 257
#define ELSE 258
#define THEN 259
#define OR 260
#define FAILED 261
#define SET 262
#define LOGFILE 263
#define FACILITY 264
#define DAEMON 265
#define SYSLOG 266
#define MAILSERVER 267
#define HTTPD 268
#define ALLOW 269
#define ADDRESS 270
#define INIT 271
#define READONLY 272
#define CLEARTEXT 273
#define MD5HASH 274
#define SHA1HASH 275
#define CRYPT 276
#define DELAY 277
#define PEMFILE 278
#define ENABLE 279
#define DISABLE 280
#define HTTPDSSL 281
#define CLIENTPEMFILE 282
#define ALLOWSELFCERTIFICATION 283
#define IDFILE 284
#define STATEFILE 285
#define SEND 286
#define EXPECT 287
#define EXPECTBUFFER 288
#define CYCLE 289
#define COUNT 290
#define REMINDER 291
#define PIDFILE 292
#define START 293
#define STOP 294
#define PATHTOK 295
#define HOST 296
#define HOSTNAME 297
#define PORT 298
#define TYPE 299
#define UDP 300
#define TCP 301
#define TCPSSL 302
#define PROTOCOL 303
#define CONNECTION 304
#define ALERT 305
#define NOALERT 306
#define MAILFORMAT 307
#define UNIXSOCKET 308
#define SIGNATURE 309
#define TIMEOUT 310
#define RESTART 311
#define CHECKSUM 312
#define EVERY 313
#define DEFAULT 314
#define HTTP 315
#define APACHESTATUS 316
#define FTP 317
#define SMTP 318
#define POP 319
#define IMAP 320
#define CLAMAV 321
#define NNTP 322
#define NTP3 323
#define MYSQL 324
#define DNS 325
#define SSH 326
#define DWP 327
#define LDAP2 328
#define LDAP3 329
#define RDATE 330
#define RSYNC 331
#define TNS 332
#define PGSQL 333
#define POSTFIXPOLICY 334
#define SIP 335
#define LMTP 336
#define STRING 337
#define PATH 338
#define MAILADDR 339
#define MAILFROM 340
#define MAILSUBJECT 341
#define MAILBODY 342
#define SERVICENAME 343
#define STRINGNAME 344
#define NUMBER 345
#define PERCENT 346
#define LOGLIMIT 347
#define CLOSELIMIT 348
#define DNSLIMIT 349
#define KEEPALIVELIMIT 350
#define REPLYLIMIT 351
#define REQUESTLIMIT 352
#define STARTLIMIT 353
#define WAITLIMIT 354
#define GRACEFULLIMIT 355
#define CLEANUPLIMIT 356
#define REAL 357
#define CHECKPROC 358
#define CHECKFILESYS 359
#define CHECKFILE 360
#define CHECKDIR 361
#define CHECKHOST 362
#define CHECKSYSTEM 363
#define CHECKFIFO 364
#define CHECKSTATUS 365
#define CHILDREN 366
#define SYSTEM 367
#define RESOURCE 368
#define MEMORY 369
#define TOTALMEMORY 370
#define LOADAVG1 371
#define LOADAVG5 372
#define LOADAVG15 373
#define MODE 374
#define ACTIVE 375
#define PASSIVE 376
#define MANUAL 377
#define CPU 378
#define TOTALCPU 379
#define CPUUSER 380
#define CPUSYSTEM 381
#define CPUWAIT 382
#define GROUP 383
#define REQUEST 384
#define DEPENDS 385
#define BASEDIR 386
#define SLOT 387
#define EVENTQUEUE 388
#define UID 389
#define GID 390
#define MMONIT 391
#define INSTANCE 392
#define USERNAME 393
#define PASSWORD 394
#define TIMESTAMP 395
#define CHANGED 396
#define SECOND 397
#define MINUTE 398
#define HOUR 399
#define DAY 400
#define SSLAUTO 401
#define SSLV2 402
#define SSLV3 403
#define TLSV1 404
#define CERTMD5 405
#define BYTE 406
#define KILOBYTE 407
#define MEGABYTE 408
#define GIGABYTE 409
#define INODE 410
#define SPACE 411
#define PERMISSION 412
#define SIZE 413
#define MATCH 414
#define NOT 415
#define IGNORE 416
#define ACTION 417
#define EXEC 418
#define UNMONITOR 419
#define ICMP 420
#define ICMPECHO 421
#define NONEXIST 422
#define INVALID 423
#define DATA 424
#define RECOVERED 425
#define PASSED 426
#define SUCCEEDED 427
#define URL 428
#define CONTENT 429
#define PID 430
#define PPID 431
#define FSFLAG 432
#define URLOBJECT 433
#define TARGET 434
#define MAXFORWARD 435
#define GREATER 436
#define LESS 437
#define EQUAL 438
#define NOTEQUAL 439
#define YYERRCODE 256
short yylhs[] = {                                        -1,
    0,    0,    1,    1,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,   17,   17,   32,   32,   32,
   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,
   19,   19,   46,   46,   46,   46,   46,   46,   46,   46,
   46,   46,   46,   46,   46,   46,   46,   21,   21,   54,
   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,
   54,   54,   54,   23,   23,   58,   58,   58,   58,   58,
   58,   58,   58,   58,   58,   58,   58,   25,   25,   59,
   59,   59,   59,   59,   59,   59,   59,   59,   59,   27,
   27,   61,   61,   61,   61,   61,   61,   29,   29,   63,
   63,   63,   63,   63,   63,   63,   63,   63,   63,   63,
   63,   31,   31,   64,   64,   64,   64,   64,    3,    3,
    3,    4,   69,   69,   14,   15,    5,    5,    5,    6,
    6,    6,   12,   13,   11,    7,   71,   71,   72,    8,
    9,   10,   76,   76,   80,   80,   79,   79,   83,   83,
   83,   83,   84,   84,   89,   89,   89,   88,   88,   88,
   88,   88,   85,   85,   93,   93,   94,   94,   86,   90,
   91,   91,   92,   92,   87,   87,   87,   87,   87,   87,
   97,   87,   98,   87,   99,   87,  100,   87,   87,   96,
   96,  101,   95,   95,   16,   16,   18,   20,   22,   24,
   26,   28,   30,   33,   33,   34,   34,  102,  102,  104,
  104,  105,  105,  106,  106,  106,  106,   81,   81,   81,
   82,   82,   77,   77,   37,   37,   38,   60,  107,  107,
  108,  115,  109,  109,  109,  109,   75,   75,   74,   74,
   74,   74,   74,  110,  110,  110,  110,  110,  110,  110,
  110,  110,  110,  110,  110,  110,  110,  110,  110,  110,
  110,  110,  110,  110,  110,  110,  110,  110,  122,  122,
  123,  123,  120,  120,  120,  121,  121,  119,  119,  119,
  118,  118,  124,  124,  124,  124,  124,  124,  124,  124,
  124,  124,   35,   36,  117,  117,  103,  103,   73,   73,
   39,  114,  114,  126,  126,   40,   40,   40,   40,   65,
  127,   66,   66,  128,  128,  128,  128,  128,  128,  128,
  128,  128,  128,  128,  128,  128,  128,  128,  128,  128,
  128,  128,  128,   67,   67,   78,   78,  129,  129,  129,
   41,   42,   42,   42,   43,   44,  130,  130,  131,   45,
  132,  132,  133,  133,  133,  133,   62,  138,  138,  139,
  139,  139,  134,  134,  140,  141,  141,  141,  135,  135,
  135,  135,  136,  137,  143,  143,  143,  142,  142,   47,
   47,  125,  125,  125,  125,  125,  125,  144,  144,  144,
  144,  144,  145,  145,  145,  145,  145,  145,  145,  112,
  146,  111,  111,  111,  147,  147,  147,  113,  113,  113,
  113,   51,   51,   51,  148,  148,  148,   55,   55,   56,
   56,   57,   70,   70,   70,   70,   70,   48,   53,   53,
   53,   53,  149,  149,   52,   52,   49,   49,   50,   50,
  116,   68,   68,   68,
};
short yylen[] = {                                         2,
    0,    1,    1,    2,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    2,    2,    2,
    2,    2,    2,    2,    2,    0,    2,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    0,    2,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    0,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    0,    2,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    0,
    2,    1,    1,    1,    1,    1,    1,    0,    2,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    0,    2,    1,    1,    1,    1,    1,    7,    4,
    8,    4,    0,    3,    4,    2,    3,    3,    5,    4,
    6,    4,    3,    3,    3,    3,    1,    2,    4,    5,
    5,    5,    1,    2,    5,    7,    0,    2,    1,    1,
    1,    1,    1,    4,    1,    2,    2,    2,    2,    3,
    3,    2,    1,    1,    2,    2,    2,    2,    2,    2,
    0,    2,    0,    1,    5,    4,    2,    3,    3,    3,
    0,    4,    0,    5,    0,    5,    0,    5,    2,    1,
    2,    1,    0,    1,    4,    4,    4,    4,    4,    4,
    2,    4,    4,    3,    4,    3,    4,    1,    2,    1,
    2,    1,    1,    2,    2,    2,    2,    0,    2,    2,
    0,    2,    0,    2,   11,   10,   10,   10,    0,    2,
    2,    2,    0,    2,    4,    2,    0,    2,    0,    1,
    1,    1,    1,    0,    3,    2,    2,    2,    2,    3,
    2,    2,    2,    2,    2,    4,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    1,    1,    2,
    2,    2,    0,    2,    2,    0,    2,    0,    2,    4,
    1,    3,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    6,    6,    0,    2,    0,    3,    0,    3,
    7,    0,    3,    1,    1,    6,    3,    7,    1,    2,
    2,    1,    2,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    0,    4,    1,    2,    1,    1,    1,
    3,    2,    2,    2,    2,    2,    1,    2,    1,    6,
    1,    2,    1,    1,    1,    1,    6,    1,    2,    1,
    1,    1,    4,    4,    4,    1,    1,    1,    4,    4,
    4,    4,    3,    3,    1,    1,    1,    1,    1,    9,
    6,    0,    1,    1,    1,    1,    1,    0,    1,    1,
    1,    1,    1,    2,    3,    1,    1,    1,    1,    1,
    1,    0,    2,    3,    0,    2,    3,    0,    6,    6,
    6,    8,   10,    7,    0,    1,    1,    8,    9,    9,
    9,    6,    0,    1,    1,    1,    1,    8,    7,    7,
    4,    4,    0,    1,    9,    6,    8,    8,    8,    8,
    2,    0,    2,    3,
};
short yydefred[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    3,    5,    6,    7,    8,    9,   10,   11,   12,
   13,   14,   15,   16,   17,   26,   41,   58,   74,    0,
  100,  108,  122,    0,    0,    0,    0,  136,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  211,    0,    0,    4,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   90,
   91,   92,   94,   95,   96,   97,   98,   99,   88,   93,
    0,  319,    0,    0,    0,    0,  137,    0,    0,    0,
  153,    0,  143,  144,    0,  145,  320,    0,    0,    0,
    0,    0,  147,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   27,   28,   29,   30,
   31,   32,   33,   34,   35,   36,   37,   38,   39,   40,
    0,    0,   43,   44,   46,   48,   47,   55,   56,   57,
   42,   45,   49,   50,   51,   52,   53,   54,    0,   60,
   61,   62,   64,   63,   68,   69,   70,   65,   66,   67,
   59,   71,   72,   73,    0,   76,   77,   79,   81,   80,
   85,   86,   87,   78,   82,   83,   84,   75,    0,    0,
  222,  223,    0,  218,    0,  321,    0,  352,  353,  354,
  355,  359,    0,  357,   89,    0,    0,    0,    0,  102,
  103,  104,  105,  106,  101,  107,  110,  111,  113,  115,
  114,  119,  120,  121,  112,  116,  117,  118,  109,    0,
  124,  125,  126,  127,  128,  123,    0,    0,  132,    0,
    0,    0,    0,    0,  154,  157,  434,  435,  436,  437,
  135,  348,  349,  350,    0,  346,    0,  142,    0,  148,
    0,    0,  326,  341,  325,  339,  343,  331,  333,  342,
  336,  340,  324,  329,  332,  335,  334,  328,  327,  337,
  338,  330,    0,  322,    0,  130,  205,  206,  208,  207,
  209,  210,  212,  213,    0,    0,    0,    0,  385,  386,
  387,    0,    0,    0,    0,  361,  363,  364,  365,  366,
    0,    0,    0,    0,    0,  444,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  214,    0,  219,  220,  216,    0,  351,  358,    0,
    0,  317,  376,  377,  378,  371,  370,    0,  368,  372,
    0,  139,    0,    0,  230,  229,    0,    0,    0,    0,
  150,    0,  151,  347,    0,  253,  250,  251,  252,    0,
    0,    0,    0,  323,    0,    0,    0,  397,  393,  394,
  395,  396,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  362,    0,  426,  427,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  240,
    0,    0,    0,    0,    0,    0,    0,  224,  226,  225,
  227,  215,  221,  217,    0,    0,    0,  369,    0,  134,
    0,  232,    0,  310,  234,    0,    0,    0,    0,    0,
    0,  158,  159,  160,  161,  162,    0,    0,  173,  174,
  141,    0,  149,  345,    0,    0,  454,  242,    0,    0,
  383,    0,  388,    0,    0,    0,    0,    0,    0,    0,
  413,    0,    0,  389,  384,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  442,  441,
    0,    0,    0,    0,  451,    0,    0,    0,    0,  241,
    0,    0,  308,    0,    0,    0,    0,    0,  155,    0,
    0,    0,    0,    0,    0,  179,  167,  176,  169,  178,
  166,  168,  175,  177,    0,    0,  172,    0,    0,  248,
    0,  129,  246,  244,    0,    0,    0,    0,    0,    0,
  279,  380,  379,  382,  381,  373,  374,    0,    0,  414,
  407,  408,  403,  406,    0,  409,    0,  410,    0,    0,
    0,    0,    0,    0,    0,  399,  400,  401,  402,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  306,    0,  314,  315,    0,    0,    0,    0,    0,
  316,    0,  375,    0,    0,    0,    0,    0,    0,    0,
  170,  171,  180,    0,    0,  131,    0,  281,  282,  256,
    0,    0,  259,  271,  270,  261,  262,  267,  268,  265,
  257,  272,  258,  263,  264,  273,  274,  275,  276,  269,
    0,  277,    0,  280,  303,  304,    0,    0,  360,    0,
    0,    0,    0,    0,    0,    0,    0,  391,  446,    0,
    0,    0,    0,  432,    0,    0,    0,    0,    0,  313,
    0,    0,  311,  318,  367,  156,    0,    0,    0,    0,
  202,    0,  200,  204,  186,  182,  184,  164,  245,    0,
  260,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  291,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  424,    0,  440,  439,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  185,  201,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  285,  284,    0,  266,    0,    0,
    0,    0,  447,  448,  449,  450,  438,    0,  422,    0,
    0,    0,  428,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  292,
  287,    0,    0,    0,    0,    0,    0,  390,  445,  429,
  431,  430,  238,  236,    0,  290,  293,  294,  295,  296,
  297,  298,  299,  300,  301,  302,  237,  416,    0,    0,
    0,    0,  423,  235,  417,  411,  419,  420,  421,
};
short yydgoto[] = {                                      10,
   11,   12,   13,   14,   15,   16,   17,   18,   19,   20,
   21,   22,   23,   24,   25,   26,   57,   27,   58,   28,
   59,   29,   60,   30,   69,   31,   83,   32,   84,   33,
   85,  117,   70,   71,  120,  121,   72,  123,   73,   74,
   75,   76,   77,   78,  130,  141,  142,  143,  144,  145,
  146,  147,  148,  161,  162,  163,  164,  178,   79,   80,
  205,  206,  219,  226,   81,  273,  107,  276,  229,  241,
  102,  103,  234,  360,  443,   90,  351,  245,  352,   91,
  232,  348,  432,  433,  434,  435,  436,  437,  438,  519,
  595,  668,  439,  440,  665,  662,  589,  657,  658,  659,
  663,  183,  322,  323,  184,  325,  317,  405,  450,  529,
  381,  547,  629,  489,  367,  402,  487,  682,  671,  685,
  728,  530,  531,  683,  373,  576,   82,  274,  246,  193,
  194,  295,  296,  297,  298,  299,  300,  338,  339,  340,
  341,  454,  301,  560,  548,  797,  764,  389,  307,
};
short yysindex[] = {                                    592,
 1327, -315, -289, -279, -264, -258, -230, -195, -155,    0,
  592,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, -171,
    0,    0,    0, -222, -137, -110,  -52,    0,  -83,  -77,
  -64,  -48,  -43,  201,   35,  -98, -117,   66,   87,   91,
   93,   39,    0,  129,  133,    0,  742, -238, 1336, 1382,
 -225,  146,  146,  -10,   81,  151,  110,  126, -171,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -115,    0, -198, 1382,  -26,  213,    0,  188, -255, -245,
    0,  179,    0,    0,  140,    0,    0,  195,  153,  187,
  256,  -98,    0,  449,  452, 1526,  286,  270,  272,  307,
  314,  322,  334,  335,  338, 1328,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -224,  263,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -246,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -147,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -271,  369,
    0,    0,  106,    0,  106,    0,  396,    0,    0,    0,
    0,    0,  126,    0,    0,  574, 1526,  286,  378,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  353,
    0,    0,    0,    0,    0,    0,  370,  433,    0,  366,
 -185,  318,  368,  417,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -116,    0,  340,    0,  181,    0,
  195, 1526,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 1248,    0,  380,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -234,   34,   34,   34,    0,    0,
    0,   34,   34,   63,  131,    0,    0,    0,    0,    0,
   34, -173,   34, -262,   34,    0,  315,  316,  -72,  299,
   34,   34,  337,  397,  434,  302,  439,  393,  407, -239,
 -136,    0, -157,    0,    0,    0, -157,    0,    0, 1526,
 1313,    0,    0,    0,    0,    0,    0,  866,    0,    0,
   34,    0,  408,  363,    0,    0,  424,  181,  365,  426,
    0,  183,    0,    0,  420,    0,    0,    0,    0,  364,
 -104, 1380,  461,    0,  481,  443,  473,    0,    0,    0,
    0,    0,  437, -216,  -79,  441,  442,  445,  445, -254,
  529,    0,  -75,    0,    0,   90,  122,  446,  480,  450,
  445,  445,  487,  458,  170,  222,  445,  459,   -6,    0,
  384,  512,  377,  463,  473,  521,  416,    0,    0,    0,
    0,    0,    0,    0, 1428,  461,  555,    0,  478,    0,
  318,    0,  364,    0,    0,  -53,  488, -243, -240,  284,
  300,    0,    0,    0,    0,    0,  197,  546,    0,    0,
    0,  489,    0,    0,  461,  286,    0,    0,  295,  202,
    0,  484,    0,  140,  494,  140,  503,  504,  568,  593,
    0,  564, -200,    0,    0,  445,  445,  445,  445,  445,
 -242,  207,  605,  606,  445,  140,  445,  445,    0,    0,
  607,  166,  511,  140,    0,  522,  256,  186,  256,    0,
  202,  611,    0,  461,  286, -200,  534,  181,    0,  567,
  569,  581,  848,    0,  583,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  584,  585,    0,  588,  645,    0,
  286,    0,    0,    0,  181,  594,  595, 1547,  256,  350,
    0,    0,    0,    0,    0,    0,    0, -200, -200,    0,
    0,    0,    0,    0,  146,    0,  672,    0,  675,  683,
  684,  685,  686,  609,  704,    0,    0,    0,    0,  445,
 -200, -200,  705,  445,  706,  707, -200,  445,  708,  445,
  445,    0,  445,    0,    0,  632,  445,  256,  668,  286,
    0,  672,    0,  364,    0,    0,    0,  642,  643,  710,
    0,    0,    0,  647,  700,    0,  364,    0,    0,    0,
  602, 1303,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  553,    0,  445,    0,    0,    0,   47,  732,    0, -200,
 -200, -200, -200, -200,  445, -200,  731,    0,    0, -200,
  734, -200, -200,    0,  735, -200,  739,  741,  745,    0,
  746,  445,    0,    0,    0,    0,  643,  643,  643,  710,
    0,  643,    0,    0,    0,    0,    0,    0,    0,  670,
    0,   34,   34,   34,   34,   34,   34,   34,   34,   34,
   34,  749,    0,   17,  571,  751,  280,  174,  672,  672,
  672,  672,  672,  752,  672, -200,    0, -200,    0,    0,
 -200,  672, -200, -200, -200, -200,  753,  643,  643,  643,
    0,    0,  701,  669,  671,  681,  682,  687,  688,  689,
  711,  712,  715, 1303,    0,    0,  716,    0, -200,  724,
  724,  724,    0,    0,    0,    0,    0, -200,    0,  672,
  672,  672,    0,  672,  672,  672,  672, -200,  678,  703,
  721,  726,  727,  728,  748,  750,  754,  756,  757,    0,
    0,  672, -250,  769,  770,  771,  672,    0,    0,    0,
    0,    0,    0,    0,  672,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  761, -200,
 -200, -200,    0,    0,    0,    0,    0,    0,    0,
};
short yyrindex[] = {                                   1046,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1051,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  303,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  523,  539, 1007, 1025,
    0,    0,    0,    0,    0,    0,    0,    0, 1093,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   40,    0, 1101, 1113, 1130, 1139,    0, 1149,  457,  937,
    0,    0,    0,    0, 1162,    0,    0,    0,    0,    0,
    5, 1175,    0,    0,    0,    0, 1183,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  656,  656,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  797,    0,
    0,    0,  228,    0,  228,    0,    0,    0,    0,    0,
    0,    0,  268,    0,    0,    0,    0,  341,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  474,    0, 1208,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, 1218,    0,   14,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  797,  762,   -2,   -2,    0,    0,
    0,  762,  762,    0,  838,    0,    0,    0,    0,    0,
   -2,  794,  762,  794,  762,    0,    0,    0,    0,    0,
  762,   -2,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  228,    0,    0,    0,  228,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  838,    0,    0,
  762,    0,    0,  457,    0,    0,    0,  531,    0,    0,
    0, 1244,    0,    0,    0,    0,    0,    0,    0,   32,
    0,    0,  303,    0,  381,    0,  -46,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  838,  838,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  838,  838,    0,    0,    0,    0,  838,    0,    0,    0,
    0, -233, -228,    0,  -46,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   40,    0,    0,    0,    0,
  474,    0, 1054,    0,    0,    0,    0,    0,    0,  836,
    0,    0,    0,    0,    0,    0,  809,    0,    0,    0,
    0,    0,    0,    0,  303, 1183,    0,    0,    0, -201,
    0,  -59,    0, 1408,  -59, 1408,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  838,  838,  838,  838,  838,
  838, -226,    0,    0,  838, -213,  838,  838,    0,    0,
    0,  838, -202, -213,    0,    0, -199,    0, -199,    0,
 -201,    0,    0,   40,  341,    0,    0,  531,    0,    0,
    0,    0,  829,  659,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  679,    0,
 1183,    0,    0,    0, -206,    0,    0,    0, -199,  -37,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  550,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  838,
    0,    0,    0,  838,    0,    0,    0,  838,    0,  838,
  838,    0,  838,    0,    0,    0,  838, -199,    0,  341,
    0,  612,    0, 1054,  722,  783,  796,    0,    0,  842,
    0,    0,    0,    0,  916,    0,  -15,    0,    0,    0,
  -17,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -249,    0,  838,    0,    0,    0,  155,    0,    0,    0,
    0,    0,    0,    0,  838,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  838,    0,    0,    0,    0,    0,    0,    0,  842,
    0,  947,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  762,  762,  762,  762,  762,  762,  762,  762,  762,
  762,   -9,    0,    0,   15,    0,  193,    0,  415,  415,
  415,  415,  415,    0,  415,    0,    0,    0,    0,    0,
    0,  550,    0,    0,    0,    0,    0,  961,  981, 1068,
    0,    0,   73,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  856,
  856,  856,    0,    0,    0,    0,    0,    0,    0,  415,
  415,  550,    0,  550,  550,  550,  550,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  550,    0,    0,    0,    0,  415,    0,    0,    0,
    0,    0,    0,    0,  550,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,
};
short yygindex[] = {                                      0,
    0, 1106,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   -8,  374,    0,    0, 1062,    0,  485,  598,
  633,  382,  788, 1274,    0,    0,    3,  -12,  232,  278,
    0,    0,    0,    0,    0,    0,    0,    0, 1055,    0,
    0,    0,    0,    0, 1125, -174,  -74, -169,    0, -366,
    0, 1026, -100, -330, -401,    0,    0,  878,    0, 1041,
  790,  714,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  472,  -18,    0,    0,    0,    0,
 -404,  -63, -112, -182, -181, -307,    0,    0,  747,  646,
 -308, 1267, -565,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  610,  412,  -58,    0,    0, -231, -221,    0,
  957,    0,  858,    0, -172,    0, -165,    0,  825,    0,
    0, -124,    0,    0, -142, -103,  -36,  860, 1033,
};
#define YYTABLESIZE 2015
short yytable[] = {                                     185,
  249,  324,  327,  324,  309,  106,  198,  197,  353,  283,
  505,  384,  385,  249,  309,  413,  655,  423,  131,  413,
  444,  499,  331,  354,  314,  305,  336,   48,  332,  417,
  312,  247,  398,  337,  461,  179,  302,  507,  788,  344,
  509,  364,  230,   86,  554,  433,  158,  175,  118,  133,
  150,  166,  249,   49,   62,   63,  389,  254,  199,  309,
  283,  314,  174,   50,  233,  508,   43,   64,  510,  459,
  460,  216,  326,  366,   65,  207,  305,  362,   51,  249,
  249,  312,  473,  474,   52,   61,  215,  533,  481,  535,
  462,   89,  541,  542,  789,  283,  249,  408,  180,  364,
  384,  385,  380,  249,  543,  409,   43,   64,  254,  564,
  544,  305,   53,  309,   65,   87,  312,  571,  398,  180,
  180,   62,   63,  733,  734,  735,  736,  737,  452,  739,
  364,  433,  391,   43,   64,   66,  743,  231,  249,  354,
  453,   65,  389,  254,   67,  309,   68,   54,  315,  310,
  392,  345,  319,  346,  404,  415,  316,  549,  550,  551,
  552,  553,  555,  311,  312,  336,  563,  584,  565,  566,
  303,  304,  337,  569,  768,  769,  770,  132,  771,  772,
  773,  774,  656,  364,   67,  283,   68,   55,  305,  104,
  306,  104,  405,  316,  597,  669,  787,  180,  249,  389,
  410,  793,   66,  389,  389,  389,  389,   88,  411,  794,
  412,   67,  243,   68,  414,  386,  387,  545,  546,  500,
  501,  278,  502,  242,  243,  244,   89,  307,  374,  375,
  220,  320,  321,  376,  377,  242,  243,  244,  388,  243,
  243,  288,  383,  247,  390,   92,  394,  303,  313,  255,
  456,  637,  398,  399,   93,  641,  243,  712,  465,  645,
   94,  647,  648,  243,  649,  455,  309,  356,  651,  464,
  247,  247,  278,  286,  484,  249,  522,  453,   43,   64,
   95,  453,  419,  503,  504,  389,   65,  247,  446,   96,
  159,  176,  288,  247,  247,   97,  344,  105,  243,  196,
  255,  344,  344,  712,  712,  712,  389,  278,  113,  389,
  389,  389,  389,  389,  686,  217,  386,  387,  389,  389,
  389,  389,  389,   98,  286,  581,  694,  288,  186,  247,
  344,  289,  344,  344,  101,  255,  160,  177,  483,  388,
  452,  495,  392,  707,  344,  344,  389,  389,  389,  389,
  453,  596,  344,  725,  392,  726,   67,  108,   68,  286,
  109,  218,  309,  309,  309,  309,  309,  309,  309,  309,
  521,  249,  249,  249,  249,  249,  249,  249,  249,  413,
  453,  110,  289,  181,  182,  111,  573,  112,  577,  247,
  247,  247,  247,  247,  247,  247,  247,  344,  344,  344,
  344,  344,  344,  344,  344,  309,  309,  309,  309,  309,
  654,  404,  404,  344,  418,  319,  404,  289,  249,  580,
   99,  100,  344,  114,  344,  187,  466,  115,  623,  368,
  119,  134,  151,  167,  467,  320,  321,  309,  127,  138,
  155,  171,  181,  182,  687,  324,  249,  404,  404,  405,
  405,  426,  427,  191,  405,  344,  228,  208,  468,  404,
  404,  428,  429,  430,  247,  212,  469,  404,  192,  369,
  370,  371,  372,  231,  515,  380,  227,  652,  516,  517,
  228,  627,  181,  182,  307,  405,  405,  526,  527,  307,
  247,  431,  378,  379,  320,  321,  286,  405,  405,  287,
  288,  289,  290,  291,  528,  405,  477,  478,  292,  293,
  380,  568,  404,  404,  404,  404,  404,  404,  404,  404,
  307,  307,   18,  236,  356,  188,  189,  190,  404,  356,
  249,  248,  307,  307,  242,  243,  244,  404,   19,  404,
  307,  124,  135,  152,  168,  237,  238,  239,  240,  418,
  405,  405,  405,  405,  405,  405,  405,  405,  479,  480,
  356,  356,  511,  512,  344,  233,  405,  200,  209,  221,
  404,  251,  356,  356,  252,  405,  275,  405,  513,  514,
  356,  356,  357,  358,  359,  307,  307,  307,  307,  307,
  307,  307,  307,  344,  523,  524,  525,  452,  730,  731,
  732,  307,  452,  556,  557,  558,  559,  277,  405,  278,
  307,  418,  307,  714,  715,  716,  717,  718,  719,  720,
  721,  722,  723,  574,  575,  356,  356,  356,  356,  356,
  356,  356,  356,  452,  452,  526,  527,  453,  708,  709,
  710,  356,  453,  307,  279,  452,  452,  796,  796,  796,
  356,  280,  356,  452,  125,  136,  153,  169,  187,  281,
  344,  344,  344,  344,  344,  344,  344,  344,  320,  321,
  282,  418,  283,  453,  453,  284,  418,  306,  181,  318,
  201,  210,  222,  356,  328,  453,  453,  798,  799,  126,
  137,  154,  170,  453,  765,  766,  330,  180,  452,  452,
  452,  452,  452,  452,  452,  452,  342,  418,  418,  343,
  344,  347,  349,  350,  452,  202,  211,  223,  228,  418,
  418,  188,  180,  452,  365,  452,  355,  418,  395,  396,
  397,  391,  401,  400,  403,  231,  404,  406,  453,  453,
  453,  453,  453,  453,  453,  453,  287,  288,  289,  290,
  291,  407,  420,  228,  453,  231,  452,  333,  334,  335,
  422,  424,  425,  453,  441,  453,  228,  104,  442,  447,
  231,  449,  418,  418,  418,  418,  418,  418,  418,  418,
  448,  451,  189,  231,   18,  457,  458,  463,  418,  380,
  470,  471,  249,  228,  472,  190,  453,  418,  475,  418,
   19,  486,  476,  482,  485,  488,  418,  490,  163,  492,
  231,  418,  493,  496,  228,  228,  228,  228,  228,  228,
  228,  228,  497,  518,  506,  520,  538,  249,  199,  532,
  418,  231,  231,  231,  231,  231,  231,  231,  231,  534,
  249,  203,  418,  418,  128,  139,  156,  172,  536,  537,
  228,  539,  540,    1,  418,  418,  570,  228,  228,  228,
  228,  228,  418,  561,  562,  567,  572,  249,  418,  579,
  203,  213,  224,  418,  231,  231,  231,  231,  231,  583,
   18,   18,   18,   18,   18,   18,   18,   18,  249,  249,
  249,  249,  249,  249,  249,  249,   19,   19,   19,   19,
   19,   19,   19,   19,  585,  588,  586,  418,  418,  418,
  418,  418,  418,  418,  418,  183,  418,  418,  587,  590,
  187,  591,  592,  418,  418,  593,  594,  187,  187,  628,
  598,  599,  418,  630,  418,  249,  309,  187,  187,  187,
  181,  631,  632,  633,  634,  635,  192,  181,  181,    2,
    3,    4,    5,    6,    7,    8,    9,  181,  181,  181,
  194,  181,  636,  640,  642,  643,  646,  187,  650,  418,
  418,  418,  418,  418,  418,  418,  418,  653,  660,  661,
  196,  664,  667,  188,  666,  670,  684,  181,  688,  696,
  188,  188,  698,  701,  418,  191,  418,  703,  116,  704,
  188,  188,  188,  705,  706,  727,   20,  713,  724,  729,
  738,  748,  749,  750,  776,  751,  187,  187,  187,  187,
  187,  187,  187,  187,   21,  752,  753,  790,  791,  792,
  188,  754,  755,  756,   62,   63,  181,  181,  181,  181,
  181,  181,  181,  181,  189,    1,   43,   64,  777,  795,
    2,  189,  189,  247,   65,  757,  758,  190,  193,  759,
  761,  189,  189,  189,  190,  190,  778,  198,  763,  443,
  163,  779,  780,  781,  190,  190,  190,  163,  163,  188,
  188,  188,  188,  188,  188,  188,  188,  163,  163,  163,
  199,  189,   22,  782,  239,  783,  412,  199,  199,  784,
   23,  785,  786,  203,  190,  425,  392,  199,  199,  199,
  203,  203,   24,  165,  415,   66,   56,  163,  122,  195,
  203,  203,  203,  195,   67,   47,   68,  250,  361,   25,
  235,  711,  197,  421,  498,  760,  578,  199,  138,  624,
  189,  189,  189,  189,  189,  189,  189,  189,  133,  329,
  203,  491,  382,  190,  190,  190,  190,  190,  190,  190,
  190,  433,  418,  393,  308,    0,  163,  163,  163,  163,
  163,  163,  163,  163,  146,    0,    0,  183,    0,    0,
    0,    0,  452,    0,  183,  183,  199,  199,  199,  199,
  199,  199,  199,  199,  183,  183,  183,    0,  309,  203,
  203,  203,  203,  203,  203,  203,  203,  233,  192,    0,
  380,    0,    0,    0,    0,  192,  192,  140,    0,    0,
    0,    0,  194,    0,  183,  192,  192,  192,    0,  194,
  194,    0,    0,  309,  287,  288,  289,  290,  291,  194,
  194,  194,  196,  152,    0,  333,  334,  335,    0,  196,
  196,    0,    0,    0,    0,  192,    0,    0,    0,  196,
  196,  196,    0,    0,    0,    0,    0,    0,   20,  194,
    0,    0,    0,  183,  183,  183,  183,  183,  183,  183,
  183,    0,    0,    0,    0,    0,   21,    0,    0,  196,
    0,    0,    0,    0,  309,  309,  309,  309,  309,  309,
  309,  309,    0,    0,  192,  192,  192,  192,  192,  192,
  192,  192,    0,    0,    0,  247,    0,    0,  194,  194,
  194,  194,  194,  194,  194,  194,    0,    0,    0,  198,
  129,  140,  157,  173,    0,    0,  198,  198,  196,  196,
  196,  196,  196,  196,  196,  196,  198,  198,  198,    0,
  247,    0,    0,    0,   22,    0,  204,  214,  225,    0,
    0,    0,   23,  247,   20,   20,   20,   20,   20,   20,
   20,   20,  363,    0,   24,    0,  198,    0,    0,    0,
    0,    0,   21,   21,   21,   21,   21,   21,   21,   21,
  247,   25,    0,    0,    0,    0,    0,    0,    0,    0,
  138,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  133,  247,  247,  247,  247,  247,  247,  247,  247,    0,
    0,    0,    0,  433,    0,  198,  198,  198,  198,  198,
  198,  198,  198,    0,    0,    0,  146,  416,    0,    0,
    0,    0,    0,    0,  452,    0,    0,    0,    0,    0,
   22,   22,   22,   22,   22,   22,   22,   22,   23,   23,
   23,   23,   23,   23,   23,   23,    0,    0,    0,  233,
   24,   24,   24,   24,   24,   24,   24,   24,    0,  140,
    0,    0,    0,    0,    0,    0,    0,   25,   25,   25,
   25,   25,   25,   25,   25,    0,  138,  138,  138,  138,
  138,  138,  138,  138,  445,  152,  133,  133,  133,  133,
  133,  133,  133,  133,    0,    0,    0,    0,    0,  433,
  433,  433,  433,  433,  433,  433,  433,    0,    0,    0,
    0,    0,  146,  146,  146,  146,  146,  146,  146,  146,
  452,  452,  452,  452,  452,  452,  452,  452,    0,    0,
    0,  253,  494,    0,    0,    0,    0,  254,    0,  255,
    0,    0,    0,    0,    0,  233,  233,  233,  233,  233,
  233,  233,  233,    0,    0,  140,  140,  140,  140,  140,
  140,  140,  140,    0,    0,    0,    0,    0,  285,   34,
    0,   35,  149,   36,   37,    0,    0,   38,    0,    0,
    0,  152,  152,  152,  152,  152,  152,  152,  152,    0,
   39,   40,    0,    0,   41,  256,  253,    0,   42,    0,
    0,    0,  254,    0,  255,    0,    0,    0,   62,   63,
    0,   43,    0,   44,    0,    0,  257,  258,  165,  259,
   43,   64,  260,    0,    0,    0,    0,    0,   65,  672,
  673,  674,  675,  676,  677,  678,  679,  680,  681,  261,
  262,    0,    0,    0,  263,  264,  433,  265,    0,  266,
  267,  268,  180,    0,   62,   63,  269,  270,  271,  272,
  256,    0,    0,  253,    0,    0,   43,   64,    0,  254,
    0,  255,    0,  286,   65,    0,  287,  288,  289,  290,
  291,  257,  258,    0,  259,  292,  293,  260,    0,   66,
    0,    0,    0,    0,   45,    0,    0,   46,   67,    0,
   68,    0,    0,  294,  261,  262,    0,    0,    0,  263,
  264,  253,  265,    0,  266,  267,  268,  254,    0,  255,
    0,  269,  270,  271,  272,    0,    0,  256,    0,    0,
    0,    0,  433,    0,    0,   66,    0,    0,    0,    0,
    0,    0,  582,    0,   67,    0,   68,    0,  257,  258,
    0,  259,    0,  433,  260,    0,  433,  433,  433,  433,
  433,    0,    0,    0,    0,  433,  433,  433,  433,  433,
    0,  261,  262,    0,    0,  256,  263,  264,    0,  265,
    0,  266,  267,  268,  625,  626,    0,    0,  269,  270,
  271,  272,    0,    0,    0,    0,  257,  258,    0,  259,
    0,    0,  260,    0,    0,    0,    0,  638,  639,  253,
    0,    0,    0,  644,    0,  254,    0,  255,    0,  261,
  262,    0,    0,    0,  263,  264,    0,  265,    0,  266,
  267,  268,    0,    0,    0,    0,  269,  270,  271,  272,
  600,  601,  602,  603,  604,  605,  606,  607,  608,  609,
  610,  611,  612,  613,  614,  615,  616,  617,  618,  619,
  620,  621,  622,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  256,    0,    0,  689,  690,  691,  692,
  693,    0,  695,    0,    0,    0,  697,    0,  699,  700,
    0,    0,  702,    0,  257,  258,    0,  259,    0,    0,
  260,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  261,  262,    0,
    0,    0,  263,  264,    0,  265,    0,  266,  267,  268,
    0,    0,    0,    0,  269,  270,  271,  272,    0,    0,
    0,    0,  740,    0,  741,    0,    0,  742,    0,  744,
  745,  746,  747,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  762,    0,    0,    0,    0,
    0,    0,    0,    0,  767,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  775,
};
short yycheck[] = {                                      63,
  101,  183,  185,  185,    0,  123,   81,  123,  125,  259,
   64,  274,  275,    0,  261,  323,  582,  348,  257,  327,
  125,  423,  197,  245,  296,  259,  199,  343,  198,  338,
  259,    0,  259,  199,  289,  261,  261,  281,  289,    0,
  281,  273,  298,  266,  287,  259,   59,   60,   57,   58,
   59,   60,  259,  343,  293,  294,  259,  259,  257,  259,
  310,  296,   60,  343,  310,  309,  305,  306,  309,  378,
  379,   84,  185,  308,  313,   84,  310,  252,  343,  286,
  287,  310,  391,  392,  343,  257,   84,  454,  397,  456,
  345,  337,  293,  294,  345,  345,  303,  337,  345,  331,
  274,  275,  345,  310,  305,  345,  305,  306,  310,  476,
  311,  345,  343,  261,  313,  338,  345,  484,  345,  345,
  345,  293,  294,  689,  690,  691,  692,  693,  345,  695,
  362,  345,  395,  305,  306,  374,  702,  393,  345,  361,
  357,  313,  345,  345,  383,  345,  385,  343,  420,  396,
  413,  337,  310,  339,    0,  330,  428,  466,  467,  468,
  469,  470,  471,  410,  411,  338,  475,  498,  477,  478,
  395,  396,  338,  482,  740,  741,  742,  416,  744,  745,
  746,  747,  584,  415,  383,  435,  385,  343,  413,  307,
  415,  307,    0,  428,  525,  597,  762,  345,  405,  259,
  337,  767,  374,  406,  407,  408,  409,  345,  345,  775,
  323,  383,  259,  385,  327,  389,  390,  418,  419,  273,
  274,  259,  276,  340,  341,  342,  337,    0,  287,  288,
  257,  389,  390,  292,  293,  340,  341,  342,  412,  286,
  287,  259,  301,  259,  303,  298,  305,  395,  396,  259,
  375,  560,  311,  312,  338,  564,  303,  662,  383,  568,
  338,  570,  571,  310,  573,  345,  262,    0,  577,  345,
  286,  287,  310,  259,  399,  262,  446,  357,  305,  306,
  345,  357,  341,  337,  338,  345,  313,  303,  363,  338,
   59,   60,  310,  262,  310,  339,  257,  415,  345,  415,
  310,  262,    0,  708,  709,  710,  366,  345,  270,  369,
  370,  371,  372,  373,  623,   84,  389,  390,  378,  379,
  380,  381,  382,  123,  310,  495,  635,  345,  339,  345,
  291,  259,  293,  294,  433,  345,   59,   60,  345,  412,
    0,  416,  345,  652,  305,  306,  406,  407,  408,  409,
  357,  521,  313,  337,  357,  339,  383,  292,  385,  345,
  295,   84,  358,  359,  360,  361,  362,  363,  364,  365,
  445,  358,  359,  360,  361,  362,  363,  364,  365,  687,
    0,  295,  310,  337,  338,  295,  487,  295,  489,  358,
  359,  360,  361,  362,  363,  364,  365,  358,  359,  360,
  361,  362,  363,  364,  365,  401,  402,  403,  404,  405,
  580,  257,  258,  374,    0,  310,  262,  345,  405,  494,
  386,  387,  383,  295,  385,  345,  337,  295,  529,  396,
   57,   58,   59,   60,  345,  389,  390,  433,   57,   58,
   59,   60,  337,  338,  627,  627,  433,  293,  294,  257,
  258,  269,  270,  344,  262,  416,    0,   84,  337,  305,
  306,  279,  280,  281,  433,   84,  345,  313,  343,  436,
  437,  438,  439,    0,  278,  345,  264,  578,  282,  283,
  293,  545,  337,  338,  257,  293,  294,  286,  287,  262,
  338,  309,  430,  431,  389,  390,  366,  305,  306,  369,
  370,  371,  372,  373,  303,  313,  337,  338,  378,  379,
  345,  346,  358,  359,  360,  361,  362,  363,  364,  365,
  293,  294,    0,  345,  257,  375,  376,  377,  374,  262,
    0,  345,  305,  306,  340,  341,  342,  383,    0,  385,
  313,   57,   58,   59,   60,  406,  407,  408,  409,    0,
  358,  359,  360,  361,  362,  363,  364,  365,  337,  338,
  293,  294,  279,  280,  262,  310,  374,   83,   84,   85,
  416,  123,  305,  306,  123,  383,  291,  385,  279,  280,
  313,  401,  402,  403,  404,  358,  359,  360,  361,  362,
  363,  364,  365,  291,  300,  301,  302,  257,  425,  426,
  427,  374,  262,  397,  398,  399,  400,  338,  416,  338,
  383,    0,  385,  672,  673,  674,  675,  676,  677,  678,
  679,  680,  681,  438,  439,  358,  359,  360,  361,  362,
  363,  364,  365,  293,  294,  286,  287,  257,  657,  658,
  659,  374,  262,  416,  338,  305,  306,  790,  791,  792,
  383,  338,  385,  313,   57,   58,   59,   60,    0,  338,
  358,  359,  360,  361,  362,  363,  364,  365,  389,  390,
  337,  257,  338,  293,  294,  338,  262,  415,    0,  311,
   83,   84,   85,  416,  289,  305,  306,  791,  792,   57,
   58,   59,   60,  313,  731,  732,  123,  345,  358,  359,
  360,  361,  362,  363,  364,  365,  337,  293,  294,  277,
  345,  394,  345,  297,  374,   83,   84,   85,  262,  305,
  306,    0,  345,  383,  345,  385,  387,  313,  414,  414,
  432,  395,  299,  337,  433,  262,  298,  345,  358,  359,
  360,  361,  362,  363,  364,  365,  369,  370,  371,  372,
  373,  345,  345,  297,  374,  393,  416,  380,  381,  382,
  337,  397,  337,  383,  345,  385,  310,  307,  405,  289,
  297,  299,  358,  359,  360,  361,  362,  363,  364,  365,
  338,  345,    0,  310,  262,  345,  345,  259,  374,  345,
  345,  312,  262,  337,  345,    0,  416,  383,  312,  385,
  262,  290,  345,  345,  421,  429,  257,  345,    0,  289,
  337,  262,  397,  259,  358,  359,  360,  361,  362,  363,
  364,  365,  345,  278,  337,  337,  259,  297,    0,  346,
  416,  358,  359,  360,  361,  362,  363,  364,  365,  346,
  310,    0,  293,  294,   57,   58,   59,   60,  346,  346,
  394,  259,  289,  262,  305,  306,  346,  401,  402,  403,
  404,  405,  313,  259,  259,  259,  345,  337,  257,  259,
   83,   84,   85,  262,  401,  402,  403,  404,  405,  346,
  358,  359,  360,  361,  362,  363,  364,  365,  358,  359,
  360,  361,  362,  363,  364,  365,  358,  359,  360,  361,
  362,  363,  364,  365,  338,   58,  338,  358,  359,  360,
  361,  362,  363,  364,  365,    0,  305,  306,  338,  337,
  262,  338,  338,  374,  313,  338,  282,  269,  270,  258,
  337,  337,  383,  259,  385,  405,    0,  279,  280,  281,
  262,  259,  259,  259,  259,  337,    0,  269,  270,  358,
  359,  360,  361,  362,  363,  364,  365,  279,  280,  281,
    0,  283,  259,  259,  259,  259,  259,  309,  337,  358,
  359,  360,  361,  362,  363,  364,  365,  310,  337,  337,
    0,  272,  283,  262,  338,  384,  434,  309,  257,  259,
  269,  270,  259,  259,  383,  337,  385,  259,  257,  259,
  279,  280,  281,  259,  259,  435,    0,  338,  260,  259,
  259,  259,  312,  345,  337,  345,  358,  359,  360,  361,
  362,  363,  364,  365,    0,  345,  345,  259,  259,  259,
  309,  345,  345,  345,  293,  294,  358,  359,  360,  361,
  362,  363,  364,  365,  262,    0,  305,  306,  346,  289,
    0,  269,  270,    0,  313,  345,  345,  262,  337,  345,
  345,  279,  280,  281,  269,  270,  346,    0,  345,  414,
  262,  346,  346,  346,  279,  280,  281,  269,  270,  358,
  359,  360,  361,  362,  363,  364,  365,  279,  280,  281,
  262,  309,    0,  346,  298,  346,  259,  269,  270,  346,
    0,  346,  346,  262,  309,  312,  345,  279,  280,  281,
  269,  270,    0,  278,  259,  374,   11,  309,   57,  337,
  279,  280,  281,   69,  383,    1,  385,  102,  251,    0,
   90,  660,  337,  344,  421,  724,  491,  309,    0,  530,
  358,  359,  360,  361,  362,  363,  364,  365,    0,  193,
  309,  405,  295,  358,  359,  360,  361,  362,  363,  364,
  365,    0,  338,  304,  132,   -1,  358,  359,  360,  361,
  362,  363,  364,  365,    0,   -1,   -1,  262,   -1,   -1,
   -1,   -1,    0,   -1,  269,  270,  358,  359,  360,  361,
  362,  363,  364,  365,  279,  280,  281,   -1,  262,  358,
  359,  360,  361,  362,  363,  364,  365,    0,  262,   -1,
  345,   -1,   -1,   -1,   -1,  269,  270,    0,   -1,   -1,
   -1,   -1,  262,   -1,  309,  279,  280,  281,   -1,  269,
  270,   -1,   -1,  297,  369,  370,  371,  372,  373,  279,
  280,  281,  262,    0,   -1,  380,  381,  382,   -1,  269,
  270,   -1,   -1,   -1,   -1,  309,   -1,   -1,   -1,  279,
  280,  281,   -1,   -1,   -1,   -1,   -1,   -1,  262,  309,
   -1,   -1,   -1,  358,  359,  360,  361,  362,  363,  364,
  365,   -1,   -1,   -1,   -1,   -1,  262,   -1,   -1,  309,
   -1,   -1,   -1,   -1,  358,  359,  360,  361,  362,  363,
  364,  365,   -1,   -1,  358,  359,  360,  361,  362,  363,
  364,  365,   -1,   -1,   -1,  262,   -1,   -1,  358,  359,
  360,  361,  362,  363,  364,  365,   -1,   -1,   -1,  262,
   57,   58,   59,   60,   -1,   -1,  269,  270,  358,  359,
  360,  361,  362,  363,  364,  365,  279,  280,  281,   -1,
  297,   -1,   -1,   -1,  262,   -1,   83,   84,   85,   -1,
   -1,   -1,  262,  310,  358,  359,  360,  361,  362,  363,
  364,  365,  125,   -1,  262,   -1,  309,   -1,   -1,   -1,
   -1,   -1,  358,  359,  360,  361,  362,  363,  364,  365,
  337,  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,  358,  359,  360,  361,  362,  363,  364,  365,   -1,
   -1,   -1,   -1,  262,   -1,  358,  359,  360,  361,  362,
  363,  364,  365,   -1,   -1,   -1,  262,  125,   -1,   -1,
   -1,   -1,   -1,   -1,  262,   -1,   -1,   -1,   -1,   -1,
  358,  359,  360,  361,  362,  363,  364,  365,  358,  359,
  360,  361,  362,  363,  364,  365,   -1,   -1,   -1,  262,
  358,  359,  360,  361,  362,  363,  364,  365,   -1,  262,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  358,  359,  360,
  361,  362,  363,  364,  365,   -1,  358,  359,  360,  361,
  362,  363,  364,  365,  125,  262,  358,  359,  360,  361,
  362,  363,  364,  365,   -1,   -1,   -1,   -1,   -1,  358,
  359,  360,  361,  362,  363,  364,  365,   -1,   -1,   -1,
   -1,   -1,  358,  359,  360,  361,  362,  363,  364,  365,
  358,  359,  360,  361,  362,  363,  364,  365,   -1,   -1,
   -1,  304,  125,   -1,   -1,   -1,   -1,  310,   -1,  312,
   -1,   -1,   -1,   -1,   -1,  358,  359,  360,  361,  362,
  363,  364,  365,   -1,   -1,  358,  359,  360,  361,  362,
  363,  364,  365,   -1,   -1,   -1,   -1,   -1,  261,  263,
   -1,  265,  257,  267,  268,   -1,   -1,  271,   -1,   -1,
   -1,  358,  359,  360,  361,  362,  363,  364,  365,   -1,
  284,  285,   -1,   -1,  288,  368,  304,   -1,  292,   -1,
   -1,   -1,  310,   -1,  312,   -1,   -1,   -1,  293,  294,
   -1,  305,   -1,  307,   -1,   -1,  389,  390,  257,  392,
  305,  306,  395,   -1,   -1,   -1,   -1,   -1,  313,  347,
  348,  349,  350,  351,  352,  353,  354,  355,  356,  412,
  413,   -1,   -1,   -1,  417,  418,  259,  420,   -1,  422,
  423,  424,  345,   -1,  293,  294,  429,  430,  431,  432,
  368,   -1,   -1,  304,   -1,   -1,  305,  306,   -1,  310,
   -1,  312,   -1,  366,  313,   -1,  369,  370,  371,  372,
  373,  389,  390,   -1,  392,  378,  379,  395,   -1,  374,
   -1,   -1,   -1,   -1,  388,   -1,   -1,  391,  383,   -1,
  385,   -1,   -1,  396,  412,  413,   -1,   -1,   -1,  417,
  418,  304,  420,   -1,  422,  423,  424,  310,   -1,  312,
   -1,  429,  430,  431,  432,   -1,   -1,  368,   -1,   -1,
   -1,   -1,  345,   -1,   -1,  374,   -1,   -1,   -1,   -1,
   -1,   -1,  496,   -1,  383,   -1,  385,   -1,  389,  390,
   -1,  392,   -1,  366,  395,   -1,  369,  370,  371,  372,
  373,   -1,   -1,   -1,   -1,  378,  379,  380,  381,  382,
   -1,  412,  413,   -1,   -1,  368,  417,  418,   -1,  420,
   -1,  422,  423,  424,  538,  539,   -1,   -1,  429,  430,
  431,  432,   -1,   -1,   -1,   -1,  389,  390,   -1,  392,
   -1,   -1,  395,   -1,   -1,   -1,   -1,  561,  562,  304,
   -1,   -1,   -1,  567,   -1,  310,   -1,  312,   -1,  412,
  413,   -1,   -1,   -1,  417,  418,   -1,  420,   -1,  422,
  423,  424,   -1,   -1,   -1,   -1,  429,  430,  431,  432,
  314,  315,  316,  317,  318,  319,  320,  321,  322,  323,
  324,  325,  326,  327,  328,  329,  330,  331,  332,  333,
  334,  335,  336,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  368,   -1,   -1,  630,  631,  632,  633,
  634,   -1,  636,   -1,   -1,   -1,  640,   -1,  642,  643,
   -1,   -1,  646,   -1,  389,  390,   -1,  392,   -1,   -1,
  395,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  412,  413,   -1,
   -1,   -1,  417,  418,   -1,  420,   -1,  422,  423,  424,
   -1,   -1,   -1,   -1,  429,  430,  431,  432,   -1,   -1,
   -1,   -1,  696,   -1,  698,   -1,   -1,  701,   -1,  703,
  704,  705,  706,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  729,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  738,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  748,
};
#define YYFINAL 10
#ifndef YYDEBUG
#define YYDEBUG 1
#endif
#define YYMAXTOKEN 439
#if YYDEBUG
char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"':'",0,0,0,0,0,"'@'",0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"IF","ELSE",
"THEN","OR","FAILED","SET","LOGFILE","FACILITY","DAEMON","SYSLOG","MAILSERVER",
"HTTPD","ALLOW","ADDRESS","INIT","READONLY","CLEARTEXT","MD5HASH","SHA1HASH",
"CRYPT","DELAY","PEMFILE","ENABLE","DISABLE","HTTPDSSL","CLIENTPEMFILE",
"ALLOWSELFCERTIFICATION","IDFILE","STATEFILE","SEND","EXPECT","EXPECTBUFFER",
"CYCLE","COUNT","REMINDER","PIDFILE","START","STOP","PATHTOK","HOST","HOSTNAME",
"PORT","TYPE","UDP","TCP","TCPSSL","PROTOCOL","CONNECTION","ALERT","NOALERT",
"MAILFORMAT","UNIXSOCKET","SIGNATURE","TIMEOUT","RESTART","CHECKSUM","EVERY",
"DEFAULT","HTTP","APACHESTATUS","FTP","SMTP","POP","IMAP","CLAMAV","NNTP",
"NTP3","MYSQL","DNS","SSH","DWP","LDAP2","LDAP3","RDATE","RSYNC","TNS","PGSQL",
"POSTFIXPOLICY","SIP","LMTP","STRING","PATH","MAILADDR","MAILFROM",
"MAILSUBJECT","MAILBODY","SERVICENAME","STRINGNAME","NUMBER","PERCENT",
"LOGLIMIT","CLOSELIMIT","DNSLIMIT","KEEPALIVELIMIT","REPLYLIMIT","REQUESTLIMIT",
"STARTLIMIT","WAITLIMIT","GRACEFULLIMIT","CLEANUPLIMIT","REAL","CHECKPROC",
"CHECKFILESYS","CHECKFILE","CHECKDIR","CHECKHOST","CHECKSYSTEM","CHECKFIFO",
"CHECKSTATUS","CHILDREN","SYSTEM","RESOURCE","MEMORY","TOTALMEMORY","LOADAVG1",
"LOADAVG5","LOADAVG15","MODE","ACTIVE","PASSIVE","MANUAL","CPU","TOTALCPU",
"CPUUSER","CPUSYSTEM","CPUWAIT","GROUP","REQUEST","DEPENDS","BASEDIR","SLOT",
"EVENTQUEUE","UID","GID","MMONIT","INSTANCE","USERNAME","PASSWORD","TIMESTAMP",
"CHANGED","SECOND","MINUTE","HOUR","DAY","SSLAUTO","SSLV2","SSLV3","TLSV1",
"CERTMD5","BYTE","KILOBYTE","MEGABYTE","GIGABYTE","INODE","SPACE","PERMISSION",
"SIZE","MATCH","NOT","IGNORE","ACTION","EXEC","UNMONITOR","ICMP","ICMPECHO",
"NONEXIST","INVALID","DATA","RECOVERED","PASSED","SUCCEEDED","URL","CONTENT",
"PID","PPID","FSFLAG","URLOBJECT","TARGET","MAXFORWARD","GREATER","LESS",
"EQUAL","NOTEQUAL",
};
char *yyrule[] = {
"$accept : cfgfile",
"cfgfile :",
"cfgfile : statement_list",
"statement_list : statement",
"statement_list : statement_list statement",
"statement : setalert",
"statement : setdaemon",
"statement : setlog",
"statement : seteventqueue",
"statement : setmmonits",
"statement : setmailservers",
"statement : setmailformat",
"statement : sethttpd",
"statement : setpid",
"statement : setidfile",
"statement : setstatefile",
"statement : setexpectbuffer",
"statement : setinit",
"statement : checkproc optproclist",
"statement : checkfile optfilelist",
"statement : checkfilesys optfilesyslist",
"statement : checkdir optdirlist",
"statement : checkhost opthostlist",
"statement : checksystem optsystemlist",
"statement : checkfifo optfifolist",
"statement : checkstatus optstatuslist",
"optproclist :",
"optproclist : optproclist optproc",
"optproc : start",
"optproc : stop",
"optproc : pid",
"optproc : ppid",
"optproc : connection",
"optproc : connectionunix",
"optproc : timeout",
"optproc : alert",
"optproc : every",
"optproc : mode",
"optproc : group",
"optproc : depend",
"optproc : resourceprocess",
"optfilelist :",
"optfilelist : optfilelist optfile",
"optfile : start",
"optfile : stop",
"optfile : timestamp",
"optfile : timeout",
"optfile : every",
"optfile : alert",
"optfile : permission",
"optfile : uid",
"optfile : gid",
"optfile : checksum",
"optfile : size",
"optfile : match",
"optfile : mode",
"optfile : group",
"optfile : depend",
"optfilesyslist :",
"optfilesyslist : optfilesyslist optfilesys",
"optfilesys : start",
"optfilesys : stop",
"optfilesys : timeout",
"optfilesys : every",
"optfilesys : alert",
"optfilesys : permission",
"optfilesys : uid",
"optfilesys : gid",
"optfilesys : mode",
"optfilesys : group",
"optfilesys : depend",
"optfilesys : inode",
"optfilesys : space",
"optfilesys : fsflag",
"optdirlist :",
"optdirlist : optdirlist optdir",
"optdir : start",
"optdir : stop",
"optdir : timestamp",
"optdir : timeout",
"optdir : every",
"optdir : alert",
"optdir : permission",
"optdir : uid",
"optdir : gid",
"optdir : mode",
"optdir : group",
"optdir : depend",
"opthostlist : opthost",
"opthostlist : opthostlist opthost",
"opthost : start",
"opthost : stop",
"opthost : connection",
"opthost : icmp",
"opthost : timeout",
"opthost : alert",
"opthost : every",
"opthost : mode",
"opthost : group",
"opthost : depend",
"optsystemlist :",
"optsystemlist : optsystemlist optsystem",
"optsystem : timeout",
"optsystem : alert",
"optsystem : every",
"optsystem : group",
"optsystem : depend",
"optsystem : resourcesystem",
"optfifolist :",
"optfifolist : optfifolist optfifo",
"optfifo : start",
"optfifo : stop",
"optfifo : timestamp",
"optfifo : timeout",
"optfifo : every",
"optfifo : alert",
"optfifo : permission",
"optfifo : uid",
"optfifo : gid",
"optfifo : mode",
"optfifo : group",
"optfifo : depend",
"optstatuslist :",
"optstatuslist : optstatuslist optstatus",
"optstatus : timeout",
"optstatus : alert",
"optstatus : every",
"optstatus : group",
"optstatus : depend",
"setalert : SET alertmail '{' eventoptionlist '}' formatlist reminder",
"setalert : SET alertmail formatlist reminder",
"setalert : SET alertmail NOT '{' eventoptionlist '}' formatlist reminder",
"setdaemon : SET DAEMON NUMBER startdelay",
"startdelay :",
"startdelay : START DELAY NUMBER",
"setexpectbuffer : SET EXPECTBUFFER NUMBER unit",
"setinit : SET INIT",
"setlog : SET LOGFILE PATH",
"setlog : SET LOGFILE SYSLOG",
"setlog : SET LOGFILE SYSLOG FACILITY STRING",
"seteventqueue : SET EVENTQUEUE BASEDIR PATH",
"seteventqueue : SET EVENTQUEUE BASEDIR PATH SLOT NUMBER",
"seteventqueue : SET EVENTQUEUE SLOT NUMBER",
"setidfile : SET IDFILE PATH",
"setstatefile : SET STATEFILE PATH",
"setpid : SET PIDFILE PATH",
"setmmonits : SET MMONIT mmonitlist",
"mmonitlist : mmonit",
"mmonitlist : mmonitlist mmonit",
"mmonit : URLOBJECT nettimeout sslversion certmd5",
"setmailservers : SET MAILSERVER mailserverlist nettimeout hostname",
"setmailformat : SET MAILFORMAT '{' formatoptionlist '}'",
"sethttpd : SET HTTPD PORT NUMBER httpdlist",
"mailserverlist : mailserver",
"mailserverlist : mailserverlist mailserver",
"mailserver : STRING username password sslversion certmd5",
"mailserver : STRING PORT NUMBER username password sslversion certmd5",
"httpdlist :",
"httpdlist : httpdlist httpdoption",
"httpdoption : ssl",
"httpdoption : signature",
"httpdoption : bindaddress",
"httpdoption : allow",
"ssl : ssldisable",
"ssl : sslenable pemfile clientpemfile allowselfcert",
"sslenable : HTTPDSSL",
"sslenable : HTTPDSSL ENABLE",
"sslenable : ENABLE HTTPDSSL",
"ssldisable : HTTPDSSL DISABLE",
"ssldisable : DISABLE HTTPDSSL",
"ssldisable : ssldisable PEMFILE PATH",
"ssldisable : ssldisable CLIENTPEMFILE PATH",
"ssldisable : ssldisable ALLOWSELFCERTIFICATION",
"signature : sigenable",
"signature : sigdisable",
"sigenable : SIGNATURE ENABLE",
"sigenable : ENABLE SIGNATURE",
"sigdisable : SIGNATURE DISABLE",
"sigdisable : DISABLE SIGNATURE",
"bindaddress : ADDRESS STRING",
"pemfile : PEMFILE PATH",
"clientpemfile :",
"clientpemfile : CLIENTPEMFILE PATH",
"allowselfcert :",
"allowselfcert : ALLOWSELFCERTIFICATION",
"allow : ALLOW STRING ':' STRING readonly",
"allow : ALLOW '@' STRING readonly",
"allow : ALLOW PATH",
"allow : ALLOW CLEARTEXT PATH",
"allow : ALLOW MD5HASH PATH",
"allow : ALLOW CRYPT PATH",
"$$1 :",
"allow : ALLOW PATH $$1 allowuserlist",
"$$2 :",
"allow : ALLOW CLEARTEXT PATH $$2 allowuserlist",
"$$3 :",
"allow : ALLOW MD5HASH PATH $$3 allowuserlist",
"$$4 :",
"allow : ALLOW CRYPT PATH $$4 allowuserlist",
"allow : ALLOW STRING",
"allowuserlist : allowuser",
"allowuserlist : allowuserlist allowuser",
"allowuser : STRING",
"readonly :",
"readonly : READONLY",
"checkproc : CHECKPROC SERVICENAME PIDFILE PATH",
"checkproc : CHECKPROC SERVICENAME PATHTOK PATH",
"checkfile : CHECKFILE SERVICENAME PATHTOK PATH",
"checkfilesys : CHECKFILESYS SERVICENAME PATHTOK PATH",
"checkdir : CHECKDIR SERVICENAME PATHTOK PATH",
"checkhost : CHECKHOST SERVICENAME ADDRESS STRING",
"checksystem : CHECKSYSTEM SERVICENAME",
"checkfifo : CHECKFIFO SERVICENAME PATHTOK PATH",
"checkstatus : CHECKSTATUS SERVICENAME PATHTOK PATH",
"start : START argumentlist exectimeout",
"start : START argumentlist useroptionlist exectimeout",
"stop : STOP argumentlist exectimeout",
"stop : STOP argumentlist useroptionlist exectimeout",
"argumentlist : argument",
"argumentlist : argumentlist argument",
"useroptionlist : useroption",
"useroptionlist : useroptionlist useroption",
"argument : STRING",
"argument : PATH",
"useroption : UID STRING",
"useroption : GID STRING",
"useroption : UID NUMBER",
"useroption : GID NUMBER",
"username :",
"username : USERNAME MAILADDR",
"username : USERNAME STRING",
"password :",
"password : PASSWORD STRING",
"hostname :",
"hostname : HOSTNAME STRING",
"connection : IF FAILED host port type protocol nettimeout rate1 THEN action1 recovery",
"connection : IF FAILED URL URLOBJECT urloption nettimeout rate1 THEN action1 recovery",
"connectionunix : IF FAILED unixsocket type protocol nettimeout rate1 THEN action1 recovery",
"icmp : IF FAILED ICMP icmptype icmpcount nettimeout rate1 THEN action1 recovery",
"host :",
"host : HOST STRING",
"port : PORT NUMBER",
"unixsocket : UNIXSOCKET PATH",
"type :",
"type : TYPE TCP",
"type : TYPE TCPSSL sslversion certmd5",
"type : TYPE UDP",
"certmd5 :",
"certmd5 : CERTMD5 STRING",
"sslversion :",
"sslversion : SSLV2",
"sslversion : SSLV3",
"sslversion : TLSV1",
"sslversion : SSLAUTO",
"protocol :",
"protocol : PROTOCOL APACHESTATUS apache_stat_list",
"protocol : PROTOCOL DEFAULT",
"protocol : PROTOCOL DNS",
"protocol : PROTOCOL DWP",
"protocol : PROTOCOL FTP",
"protocol : PROTOCOL HTTP request",
"protocol : PROTOCOL IMAP",
"protocol : PROTOCOL CLAMAV",
"protocol : PROTOCOL LDAP2",
"protocol : PROTOCOL LDAP3",
"protocol : PROTOCOL MYSQL",
"protocol : PROTOCOL SIP target maxforward",
"protocol : PROTOCOL NNTP",
"protocol : PROTOCOL NTP3",
"protocol : PROTOCOL POSTFIXPOLICY",
"protocol : PROTOCOL POP",
"protocol : PROTOCOL SMTP",
"protocol : PROTOCOL SSH",
"protocol : PROTOCOL RDATE",
"protocol : PROTOCOL RSYNC",
"protocol : PROTOCOL TNS",
"protocol : PROTOCOL PGSQL",
"protocol : PROTOCOL LMTP",
"protocol : sendexpectlist",
"sendexpectlist : sendexpect",
"sendexpectlist : sendexpectlist sendexpect",
"sendexpect : SEND STRING",
"sendexpect : EXPECT STRING",
"target :",
"target : TARGET MAILADDR",
"target : TARGET STRING",
"maxforward :",
"maxforward : MAXFORWARD NUMBER",
"request :",
"request : REQUEST PATH",
"request : REQUEST PATH CHECKSUM STRING",
"apache_stat_list : apache_stat",
"apache_stat_list : apache_stat_list OR apache_stat",
"apache_stat : LOGLIMIT operator NUMBER PERCENT",
"apache_stat : CLOSELIMIT operator NUMBER PERCENT",
"apache_stat : DNSLIMIT operator NUMBER PERCENT",
"apache_stat : KEEPALIVELIMIT operator NUMBER PERCENT",
"apache_stat : REPLYLIMIT operator NUMBER PERCENT",
"apache_stat : REQUESTLIMIT operator NUMBER PERCENT",
"apache_stat : STARTLIMIT operator NUMBER PERCENT",
"apache_stat : WAITLIMIT operator NUMBER PERCENT",
"apache_stat : GRACEFULLIMIT operator NUMBER PERCENT",
"apache_stat : CLEANUPLIMIT operator NUMBER PERCENT",
"pid : IF CHANGED PID rate1 THEN action1",
"ppid : IF CHANGED PPID rate1 THEN action1",
"icmpcount :",
"icmpcount : COUNT NUMBER",
"exectimeout :",
"exectimeout : TIMEOUT NUMBER SECOND",
"nettimeout :",
"nettimeout : TIMEOUT NUMBER SECOND",
"timeout : IF NUMBER RESTART NUMBER CYCLE THEN TIMEOUT",
"urloption :",
"urloption : CONTENT urloperator STRING",
"urloperator : EQUAL",
"urloperator : NOTEQUAL",
"alert : alertmail '{' eventoptionlist '}' formatlist reminder",
"alert : alertmail formatlist reminder",
"alert : alertmail NOT '{' eventoptionlist '}' formatlist reminder",
"alert : noalertmail",
"alertmail : ALERT MAILADDR",
"noalertmail : NOALERT MAILADDR",
"eventoptionlist : eventoption",
"eventoptionlist : eventoptionlist eventoption",
"eventoption : ACTION",
"eventoption : CHECKSUM",
"eventoption : CONNECTION",
"eventoption : CONTENT",
"eventoption : DATA",
"eventoption : EXEC",
"eventoption : FSFLAG",
"eventoption : GID",
"eventoption : ICMP",
"eventoption : INSTANCE",
"eventoption : INVALID",
"eventoption : NONEXIST",
"eventoption : PERMISSION",
"eventoption : PID",
"eventoption : PPID",
"eventoption : RESOURCE",
"eventoption : SIZE",
"eventoption : TIMEOUT",
"eventoption : TIMESTAMP",
"eventoption : UID",
"formatlist :",
"formatlist : MAILFORMAT '{' formatoptionlist '}'",
"formatoptionlist : formatoption",
"formatoptionlist : formatoptionlist formatoption",
"formatoption : MAILFROM",
"formatoption : MAILSUBJECT",
"formatoption : MAILBODY",
"every : EVERY NUMBER CYCLE",
"mode : MODE ACTIVE",
"mode : MODE PASSIVE",
"mode : MODE MANUAL",
"group : GROUP STRINGNAME",
"depend : DEPENDS dependlist",
"dependlist : dependant",
"dependlist : dependlist dependant",
"dependant : SERVICENAME",
"resourceprocess : IF resourceprocesslist rate1 THEN action1 recovery",
"resourceprocesslist : resourceprocessopt",
"resourceprocesslist : resourceprocesslist resourceprocessopt",
"resourceprocessopt : resourcecpuproc",
"resourceprocessopt : resourcemem",
"resourceprocessopt : resourcechild",
"resourceprocessopt : resourceload",
"resourcesystem : IF resourcesystemlist rate1 THEN action1 recovery",
"resourcesystemlist : resourcesystemopt",
"resourcesystemlist : resourcesystemlist resourcesystemopt",
"resourcesystemopt : resourceload",
"resourcesystemopt : resourcemem",
"resourcesystemopt : resourcecpu",
"resourcecpuproc : CPU operator NUMBER PERCENT",
"resourcecpuproc : TOTALCPU operator NUMBER PERCENT",
"resourcecpu : resourcecpuid operator NUMBER PERCENT",
"resourcecpuid : CPUUSER",
"resourcecpuid : CPUSYSTEM",
"resourcecpuid : CPUWAIT",
"resourcemem : MEMORY operator value unit",
"resourcemem : MEMORY operator NUMBER PERCENT",
"resourcemem : TOTALMEMORY operator value unit",
"resourcemem : TOTALMEMORY operator NUMBER PERCENT",
"resourcechild : CHILDREN operator NUMBER",
"resourceload : resourceloadavg operator value",
"resourceloadavg : LOADAVG1",
"resourceloadavg : LOADAVG5",
"resourceloadavg : LOADAVG15",
"value : REAL",
"value : NUMBER",
"timestamp : IF TIMESTAMP operator NUMBER time rate1 THEN action1 recovery",
"timestamp : IF CHANGED TIMESTAMP rate1 THEN action1",
"operator :",
"operator : GREATER",
"operator : LESS",
"operator : EQUAL",
"operator : NOTEQUAL",
"operator : CHANGED",
"time :",
"time : SECOND",
"time : MINUTE",
"time : HOUR",
"time : DAY",
"action : ALERT",
"action : EXEC argumentlist",
"action : EXEC argumentlist useroptionlist",
"action : RESTART",
"action : START",
"action : STOP",
"action : UNMONITOR",
"action1 : action",
"action2 : action",
"rate1 :",
"rate1 : NUMBER CYCLE",
"rate1 : NUMBER NUMBER CYCLE",
"rate2 :",
"rate2 : NUMBER CYCLE",
"rate2 : NUMBER NUMBER CYCLE",
"recovery :",
"recovery : ELSE IF RECOVERED rate2 THEN action2",
"recovery : ELSE IF PASSED rate2 THEN action2",
"recovery : ELSE IF SUCCEEDED rate2 THEN action2",
"checksum : IF FAILED hashtype CHECKSUM rate1 THEN action1 recovery",
"checksum : IF FAILED hashtype CHECKSUM EXPECT STRING rate1 THEN action1 recovery",
"checksum : IF CHANGED hashtype CHECKSUM rate1 THEN action1",
"hashtype :",
"hashtype : MD5HASH",
"hashtype : SHA1HASH",
"inode : IF INODE operator NUMBER rate1 THEN action1 recovery",
"inode : IF INODE operator NUMBER PERCENT rate1 THEN action1 recovery",
"space : IF SPACE operator value unit rate1 THEN action1 recovery",
"space : IF SPACE operator NUMBER PERCENT rate1 THEN action1 recovery",
"fsflag : IF CHANGED FSFLAG rate1 THEN action1",
"unit :",
"unit : BYTE",
"unit : KILOBYTE",
"unit : MEGABYTE",
"unit : GIGABYTE",
"permission : IF FAILED PERMISSION NUMBER rate1 THEN action1 recovery",
"match : IF matchflagnot MATCH PATH rate1 THEN action1",
"match : IF matchflagnot MATCH STRING rate1 THEN action1",
"match : IGNORE matchflagnot MATCH PATH",
"match : IGNORE matchflagnot MATCH STRING",
"matchflagnot :",
"matchflagnot : NOT",
"size : IF SIZE operator NUMBER unit rate1 THEN action1 recovery",
"size : IF CHANGED SIZE rate1 THEN action1",
"uid : IF FAILED UID STRING rate1 THEN action1 recovery",
"uid : IF FAILED UID NUMBER rate1 THEN action1 recovery",
"gid : IF FAILED GID STRING rate1 THEN action1 recovery",
"gid : IF FAILED GID NUMBER rate1 THEN action1 recovery",
"icmptype : TYPE ICMPECHO",
"reminder :",
"reminder : REMINDER NUMBER",
"reminder : REMINDER NUMBER CYCLE",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 500
#define YYMAXDEPTH  500
#endif
#endif

#define YYINITSTACKSIZE 500

int      yydebug;
int      yynerrs;
int      yyerrflag;
int      yychar;
short   *yyssp;
YYSTYPE *yyvsp;
YYSTYPE  yyval;
YYSTYPE  yylval;

/* variables for the parser stack */
static short   *yyss;
static short   *yysslim;
static YYSTYPE *yyvs;
static int      yystacksize;
#line 1625 "p.y"


/* -------------------------------------------------------- Parser interface */


/**
 * Syntactic error routine
 *
 * This routine is automatically called by the lexer!
 */
void yyerror(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogError("%s:%i: Error: %s '%s'\n", currentfile, lineno, msg, yytext);
  cfg_errflag++;
 
  FREE(msg);
  
}

/**
 * Syntactical warning routine
 */
void yywarning(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogWarning("%s:%i: Warning: %s '%s'\n", currentfile, lineno, msg, yytext);
 
  FREE(msg);
  
}

/**
 * Argument error routine
 */
void yyerror2(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogError("%s:%i: Error: %s '%s'\n", argcurrentfile, arglineno, msg, argyytext);
  cfg_errflag++;
 
  FREE(msg);
  
}

/**
 * Argument warning routine
 */
void yywarning2(const char *s, ...) {
  
  long len;
  va_list ap;
  char *msg = NULL;

  ASSERT(s);
  
  va_start(ap,s);
  msg = Util_formatString(s, ap, &len);
  va_end(ap);
  
  LogWarning("%s:%i: Warning: %s '%s'\n", argcurrentfile, arglineno, msg, argyytext);
 
  FREE(msg);
  
}

/*
 * The Parser hook - start parsing the control file
 * Returns TRUE if parsing succeeded, otherwise FALSE
 */
int parse(char *controlfile) {

  ASSERT(controlfile);

  servicelist = tail = current = NULL;

  /*
   * Secure check the monitrc file. The run control file must have the
   * same uid as the REAL uid of this process, it must have permissions
   * no greater than 700 and it must not be a symbolic link.
   */
  if (! File_checkStat(controlfile, "control file", S_IRUSR|S_IWUSR|S_IXUSR))
    return FALSE;

  if ((yyin = fopen(controlfile,"r")) == (FILE *)NULL) {
    LogError("%s: Error: cannot open the control file '%s' -- %s\n", prog, controlfile, STRERROR);
    return FALSE;
  }

  currentfile = xstrdup(controlfile);

  /* 
   * Creation of the global service list is synchronized  
   */
  LOCK(Run.mutex)
    preparse();
    yyparse();
    fclose(yyin);
    /* Add the default general system service if not specified explicitly */
    if (!hassystem)
      createservice(TYPE_SYSTEM, xstrdup(Run.localhostname), xstrdup(""), check_system);
    /* If defined - add the last service to the service list */
    if (current) {
      addservice(current);
      FREE(current);
    }
    postparse();
  END_LOCK;

  FREE(currentfile);

  if (argyytext != NULL)
    FREE(argyytext);

  return(cfg_errflag == 0);
}


/* ----------------------------------------------------------------- Private */


/**
 * Initialize objects used by the parser.
 */
static void preparse() {
  int i;
  char localhost[STRLEN];

  /*
   * Get the localhost name
   */
  if (gethostname(localhost, sizeof(localhost)) < 0)
    snprintf(localhost, STRLEN, "%s", LOCALHOST);

  /* Set instance incarnation ID */
  time(&Run.incarnation);
  /* Reset lexer */
  buffer_stack_ptr       = 0;
  lineno                 = 1;
  arglineno              = 1;
  argcurrentfile         = NULL;
  argyytext              = NULL;
  /* Reset parser */
  Run.stopped            = FALSE;
  Run.dolog              = FALSE;
  Run.dohttpd            = FALSE;
  Run.doaction           = FALSE;
  Run.httpdsig           = TRUE;
  Run.credentials        = NULL;
  Run.httpdssl           = FALSE;
  Run.httpsslpem         = NULL;
  Run.httpsslclientpem   = NULL;
  Run.clientssl          = FALSE;
  Run.mailserver_timeout = NET_TIMEOUT;
  Run.bind_addr          = NULL;
  Run.eventlist          = NULL;
  Run.eventlist_dir      = NULL;
  Run.eventlist_slots    = -1;
  Run.system             = NULL;
  Run.expectbuffer       = STRLEN;
  Run.maillist           = NULL;
  Run.mailservers        = NULL;
  Run.MailFormat.from    = NULL;
  Run.MailFormat.subject = NULL;
  Run.MailFormat.message = NULL;
  Run.localhostname      = xstrdup(localhost);
  depend_list            = NULL;
  Run.handler_init       = TRUE;
  for (i = 0; i <= HANDLER_MAX; i++)
    Run.handler_queue[i] = 0;
  /* 
   * Initialize objects
   */
  reset_uidset();
  reset_gidset();
  reset_sizeset();
  reset_mailset();
  reset_mailserverset();
  reset_portset();
  reset_permset();
  reset_icmpset();
  reset_rateset();
  reset_filesystemset();
  reset_resourceset();
  reset_checksumset();
  reset_timestampset();
}


/*
 * Check that values are reasonable after parsing
 */
static void postparse() {
  Service_T s;

  if (cfg_errflag || ! servicelist)
    return;

  /* Check the sanity of any dependency graph */
  check_depend();

  /* Check that we do not start monit in daemon mode without having a
   * poll time */
  if (!Run.polltime && (Run.isdaemon || Run.init)) {
    LogError("%s: Error: Poll time not defined. Please define poll time in the\n control file or use the -d option when starting monit\n", prog);
    cfg_errflag++;
  }

  if (Run.logfile)
    Run.dolog = TRUE;

  for (s = servicelist; s; s = s->next) {
    /* Set the general system service shortcut */
    if (s->type == TYPE_SYSTEM)
      Run.system = s;
    if (s->type != TYPE_HOST)
	continue;
    /* Verify that a remote service has a port or an icmp list */
    if (!s->portlist && !s->icmplist) {
      LogError("%s: Error: 'check host' statement is incomplete; Please specify a port number to test\n or an icmp test at the remote host: '%s'\n", prog, s->name);
      cfg_errflag++;
    }
  }
}


/*
 * Create a new service object and add any current objects to the
 * service list.
 */
static void createservice(int type, char *name, char *value, int (*check)(Service_T s)) {

  ASSERT(name);
  ASSERT(value);

  if (current)
    addservice(current);
  else
    NEW(current);

  /* Reset the current object */
  memset(current, 0, sizeof(*current));

  NEW(current->inf);
  Util_resetInfo(current);

  /* Set default values */
  current->monitor = MONITOR_INIT;
  current->mode    = MODE_ACTIVE;
  current->name    = name;
  current->type    = type;
  current->check   = check;
  current->path    = value;

  /* Initialize general event handlers */
  addeventaction(&(current)->action_DATA,     ACTION_ALERT,     ACTION_ALERT);
  addeventaction(&(current)->action_EXEC,     ACTION_ALERT,     ACTION_ALERT);
  addeventaction(&(current)->action_INVALID,  ACTION_RESTART,   ACTION_ALERT);
  addeventaction(&(current)->action_NONEXIST, ACTION_RESTART,   ACTION_ALERT);
  addeventaction(&(current)->action_TIMEOUT,  ACTION_UNMONITOR, ACTION_ALERT);
  addeventaction(&(current)->action_PID,      ACTION_ALERT,     ACTION_IGNORE);
  addeventaction(&(current)->action_PPID,     ACTION_ALERT,     ACTION_IGNORE);
  addeventaction(&(current)->action_FSFLAG,   ACTION_ALERT,     ACTION_IGNORE);

  /* Initialize internal event handlers */
  addeventaction(&(current)->action_MONIT_START,  ACTION_START, ACTION_IGNORE);
  addeventaction(&(current)->action_MONIT_STOP,   ACTION_STOP,  ACTION_IGNORE);
  addeventaction(&(current)->action_MONIT_RELOAD, ACTION_START, ACTION_IGNORE);
  addeventaction(&(current)->action_ACTION,       ACTION_ALERT, ACTION_IGNORE);
  
  pthread_mutex_init(&current->mutex, NULL);

  gettimeofday(&current->collected, NULL);

}


/*
 * Add a service object to the servicelist
 */
static void addservice(Service_T s) {
  Service_T n;

  ASSERT(s);
 
  NEW(n);
  memcpy(n, s, sizeof(*s));
  /* Add the service to the end of the service list */
  if (tail != NULL) {
    tail->next = n;
    tail->next_conf = n;
  } else {
    servicelist = n;
    servicelist_conf = n;
  }
  tail = n;
}


/* 
 * Add a dependant entry to the current service dependant list
 *
 */
static void adddependant(char *dependant) {
  Dependant_T d; 

  ASSERT(dependant);
  
  NEW(d);
  
  if (current->dependantlist != NULL)
    d->next = current->dependantlist;

  d->dependant = dependant;
  current->dependantlist = d;

}


/*
 * Add the given mailaddress with the apropriate alert notification
 * values and mail attributes to the given mailinglist.
 */
static void addmail(char *mailto, Mail_T f, Mail_T *l, unsigned int events, unsigned int reminder) {
  Mail_T m;

  ASSERT(mailto);

  NEW(m);
  m->events   = events;
  m->to       = mailto;
  m->from     = f->from;
  m->subject  = f->subject;
  m->message  = f->message;
  m->reminder = reminder;
  
  m->next = *l;
  *l = m;

  reset_mailset();
}


/*
 * Add the given portset to the current service's portlist
 */
static void addport(Port_T port) {
  Port_T p;
  char address[STRLEN];
  
  ASSERT(port);

  NEW(p);
  p->port             = port->port;
  p->type             = port->type;
  p->socket           = port->socket;
  p->family           = port->family;
  p->action           = port->action;
  p->timeout          = port->timeout;
  p->request          = port->request;
  p->generic          = port->generic;
  p->protocol         = port->protocol;
  p->pathname         = port->pathname;
  p->hostname         = port->hostname;
  p->url_request      = port->url_request;
  p->request_checksum = port->request_checksum;
  memcpy(&p->ApacheStatus, &port->ApacheStatus, sizeof(struct apache_status));

  if (p->request_checksum) {
    cleanup_hash_string(p->request_checksum);
    if (strlen(p->request_checksum) == 32)
      p->request_hashtype = HASH_MD5;
    else if (strlen(p->request_checksum) == 40)
      p->request_hashtype = HASH_SHA1;
    else
      yyerror2("invalid checksum [%s]", p->request_checksum);
  } else
    p->request_hashtype = 0;

  if (port->family == AF_INET)
    snprintf(address, STRLEN, "INET[%s:%d]", port->hostname, port->port);
  else if (port->family == AF_UNIX)
    snprintf(address, STRLEN, "UNIX[%s]", port->pathname);
  else
    address[0] = 0;
  p->address = xstrdup(address);
  
  if (port->SSL.use_ssl == TRUE) {
    if (!have_ssl()) {
      yyerror("ssl check cannot be activated. SSL is not supported");
    } else {
      if (port->SSL.certmd5 != NULL) {
	p->SSL.certmd5 = port->SSL.certmd5;
	cleanup_hash_string(p->SSL.certmd5);
      }
      p->SSL.use_ssl = TRUE;
      p->SSL.version = port->SSL.version;
    }
  }
  p->maxforward = port->maxforward;
  p->next = current->portlist;
  current->portlist = p;
  
  reset_portset();

}


/*
 * Add a new resource object to the current service resource list
 */
static void addresource(Resource_T rr) {
  Resource_T r;

  ASSERT(rr);

  NEW(r);
  if (! Run.doprocess)
    yyerror("Cannot activate service check. The process status engine was disabled. On certain systems you must run monit as root to utilize this feature)\n");
  r->resource_id = rr->resource_id;
  r->limit       = rr->limit;
  r->action      = rr->action;
  r->operator    = rr->operator;
  r->next        = current->resourcelist;

  current->resourcelist = r;
  reset_resourceset();
}


/*
 * Add a new file object to the current service timestamp list
 */
static void addtimestamp(Timestamp_T ts, int notime) {
  Timestamp_T t;

  ASSERT(ts);

  NEW(t);
  t->operator     = ts->operator;
  t->time         = ts->time;
  t->action       = ts->action;
  t->test_changes = ts->test_changes;
  
  if (t->test_changes || notime) {
    if (! File_exist(current->path)) {
      DEBUG("%s: Debug: the path '%s' used in the TIMESTAMP statement refer to a non-existing object\n", prog, current->path);
    } else if (!(t->timestamp = File_getTimestamp(current->path, S_IFDIR|S_IFREG))) {
      yyerror2("cannot get the timestamp for '%s'", current->path);
    }
  }
  
  t->next = current->timestamplist;
  current->timestamplist = t;

  reset_timestampset();

}


/*
 * Add a new Size object to the current service size list
 */
static void addsize(Size_T ss) {
  Size_T s;
  struct stat buf;

  ASSERT(ss);

  NEW(s);
  s->operator     = ss->operator;
  s->size         = ss->size;
  s->action       = ss->action;
  s->test_changes = ss->test_changes;
  /* Get the initial size for future comparision, if the file exists */
  if (s->test_changes) {
    s->test_changes_ok = !stat(current->path, &buf);
    if (s->test_changes_ok)
      s->size = (unsigned long long)buf.st_size;
  }
 
  s->next = current->sizelist;
  current->sizelist = s;

  reset_sizeset();
}


/*
 * Set Checksum object in the current service
 */
static void addchecksum(Checksum_T cs) {

  int len;
  Checksum_T c;

  ASSERT(cs);

  cs->test_changes_ok = TRUE;

  if (!cs->hash) {
    if (cs->type == HASH_UNKNOWN)
      cs->type = DEFAULT_HASH;
    if ( !(cs->hash = Util_getChecksum(current->path, cs->type))) {
      if (cs->test_changes == TRUE) {
        /* If the file doesn't exist and we're checking for checksum changes, set dummy value */
        cs->test_changes_ok = FALSE;
        cs->hash = xstrdup("00000000000000000000000000000000");
      } else {
        yyerror2("cannot compute a checksum for file %s", current->path);
        reset_checksumset();
        return;
      }
    }
  }

  len = cleanup_hash_string(cs->hash);

  if (cs->type == HASH_UNKNOWN) {
    if (len == 32) {
      cs->type = HASH_MD5;
    } else if (len == 40) {
      cs->type = HASH_SHA1;
    } else {
      yyerror2("invalid checksum [%s] for file %s", cs->hash, current->path);
      reset_checksumset();
      return;
    }
  } else if (( cs->type == HASH_MD5 && len!=32 ) || ( cs->type == HASH_SHA1 && len != 40 )) {
    yyerror2("invalid checksum [%s] for file %s", cs->hash, current->path);
    reset_checksumset();
    return;
  }

  NEW(c);

  c->type            = cs->type;
  c->hash            = cs->hash;
  c->test_changes    = cs->test_changes;
  c->test_changes_ok = cs->test_changes_ok;
  c->action          = cs->action;
 
  current->checksum = c;

  reset_checksumset();

}


/*
 * Set Perm object in the current service
 */
static void addperm(Perm_T ps) {
  Perm_T p;

  ASSERT(ps);

  NEW(p);
  p->perm       = ps->perm;
  p->action     = ps->action;
  current->perm = p;
  reset_permset();

}

/*
 * Set Match object in the current service
 */
static void addmatch(Match_T ms, int actionnumber, int linenumber) {
  Match_T m;
  Match_T ml;
  int     reg_return;
  
  ASSERT(ms);

  NEW(m);
#ifdef HAVE_REGEX_H
  NEW(m->regex_comp);
#endif

  m->match_string = ms->match_string;
  m->match_path   = ms->match_path ? xstrdup(ms->match_path) : NULL;
  m->action       = ms->action;
  m->not          = ms->not;
  m->ignore       = ms->ignore;
  m->next         = NULL;

  addeventaction(&(m->action), actionnumber, ACTION_IGNORE);

#ifdef HAVE_REGEX_H
  reg_return = regcomp(m->regex_comp, ms->match_string, REG_NOSUB|REG_EXTENDED);

  if (reg_return != 0) {
    char errbuf[STRLEN];
    regerror(reg_return, ms->regex_comp, errbuf, STRLEN);
    if (m->match_path != NULL) 
      yyerror2("regex parsing error:%s on line %i of", errbuf, linenumber);
    else
      yyerror2("regex parsing error:%s", errbuf);
  }
#endif

  if (current->matchlist) {
    /* Find the end of the list */
    for (ml = current->matchlist; ml->next; ml = ml->next)
      ;

    ml->next = m;
    
  } else
    current->matchlist = m;
}


static void addmatchpath(Match_T ms, int actionnumber) {

  FILE *handle;
  Command_T savecommand = NULL;
  char buf[2048];
  int linenumber = 0;

  ASSERT(ms->match_path);

  handle = fopen(ms->match_path, "r");
  if (handle == NULL) {
    yyerror2("cannot read regex match file (%s)", ms->match_path);
    return;
  }
  
  while (!feof(handle)) {
    int len;

    linenumber++;
    
    if (! fgets(buf, 2048, handle))
      continue;

    len = strlen(buf);

    if (len == 0 || buf[0] == '\n')
      continue;

    if (buf[len-1] == '\n')
      buf[len-1] = 0;

    ms->match_string = xstrdup(buf);

    /* The addeventaction() called from addmatch() will reset the
     * command1 to NULL, but we need to duplicate the command for
     * each line, thus need to save it here */
    if (actionnumber == ACTION_EXEC) {
      if (command1 == NULL) {
        ASSERT(savecommand);
        command1 = savecommand;
      }
      savecommand = copycommand(command1);
    }

    addmatch(ms, actionnumber, linenumber);
  }

  if (actionnumber == ACTION_EXEC && savecommand)
    gccmd(&savecommand);

  fclose(handle);
}


/*
 * Set Uid object in the current service
 */
static void adduid(Uid_T us) {
  Uid_T u;

  ASSERT(us);

  NEW(u);
  u->uid       = us->uid;
  u->action    = us->action;
  current->uid = u;
  reset_uidset();
}


/*
 * Set Gid object in the current service
 */
static void addgid(Gid_T gs) {
  Gid_T g;

  ASSERT(gs);

  NEW(g);
  g->gid       = gs->gid;
  g->action    = gs->action;
  current->gid = g;
  reset_gidset();
}


/*
 * Add a new filesystem to the current service's filesystem list
 */
static void addfilesystem(Filesystem_T ds) {
  Filesystem_T dev;

  ASSERT(ds);
  
  NEW(dev);
  dev->resource           = ds->resource;
  dev->operator           = ds->operator;
  dev->limit_absolute     = ds->limit_absolute;
  dev->limit_percent      = ds->limit_percent;
  dev->action             = ds->action;

  dev->next               = current->filesystemlist;
  current->filesystemlist = dev;

  reset_filesystemset();

}


/*
 * Add a new icmp object to the current service's icmp list
 */
static void addicmp(Icmp_T is) {
  if (!getuid()) {
    Icmp_T icmp;

    ASSERT(is);

    NEW(icmp);
    icmp->type         = is->type;      
    icmp->count        = is->count > ICMP_MAX_COUNT ? ICMP_MAX_COUNT : is->count;     
    icmp->timeout      = is->timeout;
    icmp->action       = is->action;
    icmp->is_available = FALSE;
    icmp->response     = -1;
  
    icmp->next         = current->icmplist;
    current->icmplist  = icmp;
  } else
    yyerror("icmp statements must be run as root");
  reset_icmpset();
}


/*
 * Set EventAction object
 */
static void addeventaction(EventAction_T *_ea, int failed, int succeeded) {
  EventAction_T ea;

  ASSERT(_ea);

  NEW(ea);
  NEW(ea->failed);
  NEW(ea->succeeded);

  ea->failed->id     = failed;
  ea->failed->count  = rate1.count;
  ea->failed->cycles = rate1.cycles;
  if (failed == ACTION_EXEC) {
    ASSERT(command1);
    ea->failed->exec = command1;
    command1 = NULL;
  }
  describeAction(ea->failed);

  ea->succeeded->id     = succeeded;
  ea->succeeded->count  = rate2.count;
  ea->succeeded->cycles = rate2.cycles;
  if (succeeded == ACTION_EXEC) {
    ASSERT(command2);
    ea->succeeded->exec = command2;
    command2 = NULL;
  }
  describeAction(ea->succeeded);
  *_ea = ea;
  reset_rateset();
}


/*
 * Redefine EventAction object (used for default action overloading)
 */
static void seteventaction(EventAction_T *_ea, int failed, int succeeded) {
  EventAction_T ea = *_ea;

  ASSERT(ea);
  ASSERT(ea->failed);
  ASSERT(ea->succeeded);

  ea->failed->id     = failed;
  ea->failed->count  = rate1.count;
  ea->failed->cycles = rate1.cycles;
  if (failed == ACTION_EXEC) {
    ASSERT(command1);
    ea->failed->exec = command1;
    command1 = NULL;
  }
  describeAction(ea->failed);

  ea->succeeded->id     = succeeded;
  ea->succeeded->count  = rate2.count;
  ea->succeeded->cycles = rate2.cycles;
  if (succeeded == ACTION_EXEC) {
    ASSERT(command2);
    ea->succeeded->exec = command2;
    command2 = NULL;
  }
  describeAction(ea->succeeded);
}


/*
 * Return a protocol object for the given protocol
 */
static void *addprotocol(int protocol) {

  switch (protocol) {
  case P_APACHESTATUS:  return create_apache_status();
  case P_DNS:           return create_dns();
  case P_DWP:           return create_dwp();
  case P_FTP:           return create_ftp();
  case P_GENERIC:       return create_generic();
  case P_HTTP:          return create_http();
  case P_IMAP:          return create_imap();
  case P_CLAMAV:        return create_clamav();
  case P_LDAP2:         return create_ldap2();
  case P_LDAP3:         return create_ldap3();
  case P_MYSQL:         return create_mysql();
  case P_NNTP:          return create_nntp();
  case P_NTP3:          return create_ntp3();
  case P_POSTFIXPOLICY: return create_postfix_policy();
  case P_POP:           return create_pop();
  case P_SMTP:          return create_smtp();
  case P_SSH:           return create_ssh();
  case P_RDATE:         return create_rdate();
  case P_RSYNC:         return create_rsync();
  case P_TNS:           return create_tns();
  case P_PGSQL:         return create_pgsql();
  case P_SIP:           return create_sip();
  case P_LMTP:          return create_lmtp();
  }

  return create_default();
   
}


/*
 * Add a generic protocol handler to 
 */
static void addgeneric(Port_T port, char *send, char *expect) {
  Generic_T g = port->generic;
  
  if (g == NULL) {
    NEW(g);
    port->generic = g;
  } else {
    while (g->next != NULL)
      g = g->next;
    NEW(g->next);
    g = g->next;
  }
  
  if (send != NULL) {
    g->send = xstrdup(send);
    g->expect = NULL;
  } else if (expect != NULL) {
#ifdef HAVE_REGEX_H
    
    int   reg_return;
    NEW(g->expect);
    reg_return = regcomp(g->expect, expect, REG_NOSUB|REG_EXTENDED);
    if (reg_return != 0) {
      char errbuf[STRLEN];
      regerror(reg_return, g->expect, errbuf, STRLEN);
      yyerror2("regex parsing error:%s", errbuf);
    }
#else
    g->expect = xstrdup(expect);
#endif
    g->send = NULL;
  } 
}


/*
 * Add the current command object to the current service object's
 * start or stop program.
 */
static void addcommand(int what, unsigned timeout) {

  switch(what) {
  case START: current->start = command; break;
  case STOP:  current->stop = command; break;
  }

  command->timeout = timeout;
  
  command = NULL;
  
}

  
/*
 * Add a new argument to the argument list
 */
static void addargument(char *argument) {

  ASSERT(argument);

  if (! command) {
    
    NEW(command);
    check_exec(argument);
    
  }
  
  command->arg[command->length++] = argument;
  command->arg[command->length] = NULL;
  
  if (command->length >= ARGMAX)
    yyerror("exceeded maximum number of program arguments");
  
}


/*
 * Setup a url request for the current port object
 */
static void prepare_urlrequest(URL_T U) {

  ASSERT(U);
  
  portset.protocol = addprotocol(P_HTTP);

  if (urlrequest == NULL)
    NEW(urlrequest);
  urlrequest->url = U;
  portset.hostname = xstrdup(U->hostname);
  check_hostname(portset.hostname);
  portset.port = U->port;
  portset.url_request = urlrequest;
  portset.type = SOCK_STREAM;
  portset.request = Util_getString("%s%s%s", U->path, U->query ? "?" : "", U->query ? U->query : "");
  /* Only the HTTP protocol is supported for URLs.
     See also the lexer if this is to be changed in
     the future */
  portset.protocol = addprotocol(P_HTTP);
  if (IS(U->protocol, "https"))
    portset.SSL.use_ssl = TRUE;
  
}


/*
 * Set the url request for a port
 */
static void  seturlrequest(int operator, char *regex) {
  
  ASSERT(regex);

  if (! urlrequest)
    NEW(urlrequest);
  urlrequest->operator = operator;
#ifdef HAVE_REGEX_H
  {    
    int reg_return;
    NEW(urlrequest->regex);
    reg_return = regcomp(urlrequest->regex, regex, REG_NOSUB|REG_EXTENDED);
    if (reg_return != 0) {
      char errbuf[STRLEN];
      regerror(reg_return, urlrequest->regex, errbuf, STRLEN);
      yyerror2("regex parsing error: %s", errbuf);
    }
  }
#else
  urlrequest->regex = xstrdup(regex);
#endif

}


/*
 * Add a new data recipient server to the mmonit server list
 */
static void addmmonit(URL_T url, int timeout, int sslversion, char *certmd5) {
  Mmonit_T c;
  
  ASSERT(url);

  NEW(c);
  c->url = url;
  if (!strcmp(c->url->protocol, "https")) {
    if (!have_ssl()) {
      yyerror("ssl check cannot be activated. SSL is not supported");
    } else {
      c->ssl.use_ssl = TRUE;
      c->ssl.version = sslversion;
      if (certmd5) {
	c->ssl.certmd5 = certmd5;
	cleanup_hash_string(c->ssl.certmd5);
      }
    }
  }
  c->timeout = timeout;
  c->next = NULL;

  if (Run.mmonits) {
    Mmonit_T C;
    for (C = Run.mmonits; C->next; C = C->next)
      /* Empty */ ;
    C->next = c;
  } else {
    Run.mmonits = c;
  }
}


/*
 * Add a new smtp server to the mail server list
 */
static void addmailserver(MailServer_T mailserver) {

  MailServer_T s;
  
  ASSERT(mailserver->host);

  NEW(s);
  s->host        = mailserver->host;
  s->port        = mailserver->port;
  s->username    = mailserver->username;
  s->password    = mailserver->password;
  s->ssl.use_ssl = mailserver->ssl.use_ssl;
  s->ssl.version = mailserver->ssl.version;
  s->ssl.certmd5 = mailserver->ssl.certmd5;

  s->next = NULL;

  if (Run.mailservers) {
    MailServer_T l;
    for (l = Run.mailservers; l->next; l = l->next) /* empty */;
    l->next = s;
  } else {
    Run.mailservers = s;
  }
  
  reset_mailserverset();
}


/*
 * Return uid if found on the system. If the parameter user is NULL
 * the uid parameter is used for looking up the user id on the system,
 * otherwise the user parameter is used.
 */
static uid_t get_uid(char *user, uid_t uid) {
  struct passwd *pwd;

  if (user) {
    pwd = getpwnam(user);

    if (pwd == NULL) {
      yyerror2("requested user not found on the system");
      return(0);
    }

  } else {

    if ( (pwd = getpwuid(uid)) == NULL ) {
      yyerror2("requested uid not found on the system");
      return(0);
    }
  }

  return(pwd->pw_uid);

}


/*
 * Return gid if found on the system. If the parameter group is NULL
 * the gid parameter is used for looking up the group id on the system,
 * otherwise the group parameter is used.
 */
static gid_t get_gid(char *group, gid_t gid) {
  struct group *grd;

  if (group) {
    grd = getgrnam(group);

    if (grd == NULL) {
      yyerror2("requested group not found on the system");
      return(0);
    }

  } else {

    if ( (grd = getgrgid(gid)) == NULL ) {
      yyerror2("requested gid not found on the system");
      return(0);
    }

  }

  return(grd->gr_gid);

}


/*
 * Add a new user id to the current command object.
 */
static void addeuid(uid_t uid) {
  if (!getuid()) {
    command->has_uid = TRUE;
    command->uid = uid;
  } else
    yyerror("uid statement requires root privileges");
}


/*
 * Add a new group id to the current command object.
 */
static void addegid(gid_t gid) {
  if (!getuid()) {
    command->has_gid = TRUE;
    command->gid = gid;
  } else
    yyerror("gid statement requires root privileges");
}


/*
 * Reset the logfile if changed
 */
static void setlogfile(char *logfile) {
  if (Run.logfile) {
    if (IS(Run.logfile, logfile)) {
      FREE(logfile);
      return;
    } else
      FREE(Run.logfile);
  }
  Run.logfile = logfile;
}


/*
 * Reset the pidfile if changed
 */
static void setpidfile(char *pidfile) {
  if (Run.pidfile) {
    if (IS(Run.pidfile, pidfile)) {
      FREE(pidfile);
      return;
    } else
      FREE(Run.pidfile);
  }
  Run.pidfile = pidfile;
}


/*
 * Read a apache htpasswd file and add credentials found for username
 */
static void addhtpasswdentry(char *filename, char *username, int dtype) {
  char *ht_username = NULL;
  char *ht_passwd = NULL;
  char buf[STRLEN];
  FILE *handle = NULL;
  int credentials_added = 0;
  
  ASSERT(filename);

  handle = fopen(filename, "r");

  if ( handle == NULL ) {
    if (username != NULL)
      yyerror2("cannot read htpasswd (%s)", filename);
    else
      yyerror2("cannot read htpasswd", filename);
    return;
  }
  
  while (!feof(handle)) {
    char *colonindex = NULL;
    int i;
    
    if (! fgets(buf, STRLEN, handle))
      continue;

    /* strip trailing non visible characters */
    for (i = strlen(buf)-1; i >= 0; i--) {
      if ( buf[i] == ' '  || buf[i] == '\r' || buf[i] == '\n' || buf[i] == '\t' )
        buf[i] ='\0';
      else
        break;
    }

    if ( NULL == (colonindex = strchr(buf, ':')))
      continue;

    ht_passwd = xstrdup(colonindex+1);
    *colonindex = '\0';

    /* In case we have a file in /etc/passwd or /etc/shadow style we
     *  want to remove ":.*$" and Crypt and MD5 hashed dont have a colon
     */ 
    
    if ( (NULL != (colonindex = strchr(ht_passwd, ':'))) && ( dtype != DIGEST_CLEARTEXT) )
      *colonindex = '\0';

    ht_username = xstrdup(buf);

    if (username == NULL) {
      if (addcredentials(ht_username, ht_passwd, dtype, FALSE))
        credentials_added++;
    } else if (strcmp(username, ht_username) == 0)  {
      if (addcredentials(ht_username, ht_passwd, dtype, FALSE))
        credentials_added++;
    } else {
      FREE(ht_passwd);
      FREE(ht_username);
    }
  }

  if (credentials_added == 0) {
    if ( username == NULL )
      yywarning2("htpasswd file (%s) has no usable credentials", filename);
    else
      yywarning2("htpasswd file (%s) has no usable credentials for user %s", filename, username);
  }
  fclose(handle);
}


#ifdef HAVE_LIBPAM
static void addpamauth(char* groupname, int readonly) {
  Auth_T c = NULL;
  Auth_T prev = NULL;

  ASSERT(groupname);

  if (Run.credentials == NULL)
    NEW(Run.credentials);

  c = Run.credentials;
  do {
    if (c->groupname != NULL && IS(c->groupname, groupname)) {
      yywarning2("PAM group %s was added already, entry ignored", groupname);
      FREE(groupname);
      return;
    }
    prev = c;
    c = c->next;
  } while (c != NULL);

  NEW(prev->next);
  c = prev->next;

  c->next        = NULL;
  c->uname       = NULL;
  c->passwd      = NULL;
  c->groupname   = groupname;
  c->digesttype  = DIGEST_PAM;
  c->is_readonly = readonly;
  
  DEBUG("%s: Adding PAM group '%s'.\n", prog, groupname); 

  return;
}
#endif


/*
 * Add Basic Authentication credentials
 */
static int addcredentials(char *uname, char *passwd, int dtype, int readonly) {
  Auth_T c;

  ASSERT(uname);
  ASSERT(passwd);

  if (Run.credentials == NULL) {
    NEW(Run.credentials);
    c = Run.credentials;
  } else {

    if (Util_getUserCredentials(uname) != NULL) {
      yywarning2("credentials for user %s were already added, entry ignored", uname);
      FREE(uname);
      FREE(passwd);
      return FALSE;
    }

    c = Run.credentials;

    while ( c->next != NULL )
      c = c->next;

    NEW(c->next);
    c = c->next;
        
  }
  
  c->next        = NULL;
  c->uname       = uname;
  c->passwd      = passwd;
  c->groupname   = NULL;
  c->digesttype  = dtype;
  c->is_readonly = readonly;
  
  DEBUG("%s: Debug: Adding credentials for user '%s'.\n", prog, uname); 
  
  return TRUE;
  
}


/*
 * Set the syslog and the facilities to be used
 */
static void setsyslog(char *facility) {

  if (!Run.logfile || ihp.logfile) {
    ihp.logfile = TRUE;
    setlogfile(xstrdup("syslog"));
    Run.use_syslog = TRUE;
    Run.dolog = TRUE;
  }

  if (facility) {
    if (IS(facility,"log_local0"))
      Run.facility = LOG_LOCAL0;
    else if (IS(facility, "log_local1"))
      Run.facility = LOG_LOCAL1;
    else if (IS(facility, "log_local2"))
      Run.facility = LOG_LOCAL2;
    else if (IS(facility, "log_local3"))
      Run.facility = LOG_LOCAL3;
    else if (IS(facility, "log_local4"))
      Run.facility = LOG_LOCAL4;
    else if (IS(facility, "log_local5"))
      Run.facility = LOG_LOCAL5;
    else if (IS(facility, "log_local6"))
      Run.facility = LOG_LOCAL6;
    else if (IS(facility, "log_local7"))
      Run.facility = LOG_LOCAL7;
    else if (IS(facility, "log_daemon"))
      Run.facility = LOG_DAEMON;
    else
      yyerror2("invalid syslog facility");
  } else {
    Run.facility = LOG_USER;
  }
  
}


/*
 * Reset the current mailset, eventset and reminder for reuse
 */
static void reset_mailset() {
  memset(&mailset, 0, sizeof(struct mymail));
  eventset = EVENT_NULL;
}


/*
 * Reset the mailserver set to default values
 */
static void reset_mailserverset() {
  memset(&mailserverset, 0, sizeof(struct mymailserver));
  mailserverset.port = PORT_SMTP;
  mailserverset.ssl.use_ssl = FALSE;
  mailserverset.ssl.version = SSL_VERSION_AUTO;
}


/*
 * Reset the Port set to default values
 */
static void reset_portset() {
  memset(&portset, 0, sizeof(struct myport));
  portset.socket = -1;
  portset.type = SOCK_STREAM;
  portset.family = AF_INET;
  portset.SSL.version = SSL_VERSION_AUTO;
  portset.timeout = NET_TIMEOUT;
  portset.maxforward = 70;
  urlrequest = NULL;
}


/*
 * Reset the Proc set to default values
 */
static void reset_resourceset() {
  resourceset.resource_id = 0;
  resourceset.limit = 0;
  resourceset.action = NULL;
  resourceset.operator = OPERATOR_EQUAL;
}


/*
 * Reset the Timestamp set to default values
 */
static void reset_timestampset() {
  timestampset.operator = OPERATOR_EQUAL;
  timestampset.time = 0;
  timestampset.test_changes = FALSE;
  timestampset.action = NULL;
}


/*
 * Reset the Size set to default values
 */
static void reset_sizeset() {
  sizeset.operator = OPERATOR_EQUAL;
  sizeset.size = 0;
  sizeset.test_changes = FALSE;
  sizeset.action = NULL;
}


/*
 * Reset the Checksum set to default values
 */
static void reset_checksumset() {
  checksumset.type = HASH_UNKNOWN;
  checksumset.hash = NULL;
  checksumset.test_changes = FALSE;
  checksumset.action = NULL;
}


/*
 * Reset the Perm set to default values
 */
static void reset_permset() {
  permset.perm = 0;
  permset.action = NULL;
}


/*
 * Reset the Uid set to default values
 */
static void reset_uidset() {
  uidset.uid = 0;
  uidset.action = NULL;
}


/*
 * Reset the Gid set to default values
 */
static void reset_gidset() {
  gidset.gid = 0;
  gidset.action = NULL;
}


/*
 * Reset the Filesystem set to default values
 */
static void reset_filesystemset() {
  filesystemset.resource = 0;
  filesystemset.operator = OPERATOR_EQUAL;
  filesystemset.limit_absolute = -1;
  filesystemset.limit_percent = -1;
  filesystemset.action = NULL;
}


/*
 * Reset the ICMP set to default values
 */
static void reset_icmpset() {
  icmpset.type = ICMP_ECHO;
  icmpset.count = ICMP_ATTEMPT_COUNT;
  icmpset.timeout = NET_TIMEOUT;
  icmpset.action = NULL;
}


/*
 * Reset the Rate set to default values
 */
static void reset_rateset() {
  rate1.count  = 1;
  rate1.cycles = 1;

  rate2.count  = 1;
  rate2.cycles = 1;
}


/* ---------------------------------------------------------------- Checkers */


/*
 * Check for unique service name
 */
static void check_name(char *name) {

  ASSERT(name);

  if (Util_existService(name) || (current && IS(name, current->name)))
    yyerror2("service name conflict, %s already defined", name);
  if (name && *name == '/')
    yyerror2("service name must not start with '/' -- ", name);
  
}


/*
 * Permission statement semantic check
 */
static int check_perm(int perm) {
  long result;
  char *status;
  char buf[STRLEN];

  snprintf(buf, STRLEN, "%d", perm);

  result = strtol(buf, &status, 8);

  if ( *status != '\0' || result < 0 || result > 07777 )
    yyerror2("permission statements must have an octal value between 0 and 7777");

  return result;
}


/*
 * Timeout statement semantic check
 */
static void check_timeout(int s, int c) {
  if (s > c)
    yyerror2("the number of restarts must be less than poll cycles");
  if (s <= 0 || c <= 0)
    yyerror2("zero or negative values not allowed in a timeout statement");
}


/*
 * Every statement semantic check
 */
static void check_every(int every) {
  if (every <= 1)
    yyerror2("an EVERY statement must have a value greater than 1");
}


/*
 * Check hostname 
 */
static void check_hostname(char *hostname) {

  ASSERT(hostname);

  if (!check_host(hostname))
    yywarning2("hostname did not resolve");
}

/*
 * Check the dependency graph for errors
 * by doing a topological sort, thereby finding any cycles.
 * Assures that graph is a Directed Acyclic Graph (DAG).
 */
static void check_depend() {
  Service_T s;
  Service_T depends_on = NULL;
  Service_T* dlt = &depend_list; /* the current tail of it                                 */
  int done;                      /* no unvisited nodes left?                               */
  int found_some;                /* last iteration found anything new ?                    */
  depend_list = NULL;            /* depend_list will be the topological sorted servicelist */

  do { 
    done = TRUE;
    found_some = FALSE; 
    for (s = servicelist; s; s = s->next) {
        Dependant_T d;
      if (s->visited)
	  continue;
      done = FALSE; // still unvisited nodes
      depends_on = NULL;
      for (d = s->dependantlist; d; d = d->next) {
        Service_T dp = Util_getService(d->dependant);
        if (!dp) {
          LogError("%s: Error: Depend service '%s' is not defined in the control file\n", prog, d->dependant);
          exit(1);
        }
        if (!dp->visited) {
          depends_on = dp;
        }
      }

      if (!depends_on) {
        s->visited = TRUE;
        found_some = TRUE;
        *dlt = s;
        dlt = &s->next_depend;
      }
    }
  } while (found_some && !done);
	
  if (!done) {
        ASSERT(depends_on);
	LogError("%s: Error: Found a depend loop in the control file involving the service '%s'\n", prog, depends_on->name);
	exit(1);
   } 

  ASSERT(depend_list);
  servicelist = depend_list;
    
  for (s = depend_list; s; s = s->next_depend)
    s->next = s->next_depend;
    
  reset_depend();
}


/*
 * Check if the executable exist
 */
static void check_exec(char *exec) {
  if (! File_exist(exec))
    yywarning2("the executable does not exist");
}
 
 
/* Return a valid max forward value for SIP header */
static int verifyMaxForward(int mf) { 
  int max = 70;
  
  if (mf >= 0 && mf <= 255)
    max = mf;
  else
    yywarning2("SIP max forward is outside the range [0..255]. Setting max forward to 70");
  
  return max;
}


/* -------------------------------------------------------------------- Misc */


/*
 * Cleans up an md5 string, tolower and remove byte separators
 */
static int cleanup_hash_string(char *hashstring) {
  int i = 0, j = 0;

  ASSERT(hashstring);

  while (hashstring[i] != '\0') {
    if (isxdigit((int) hashstring[i])) {
      hashstring[j] = tolower((int)hashstring[i]);
      j++;
    } 
    i++;
  }
  hashstring[j] = '\0';
  return j;
}


static void describeAction(Action_T A) {
  #define BUF_CURSOR    (A->description + strlen(A->description))
  #define BUF_AVAILABLE (sizeof(A->description) - strlen(A->description))
  snprintf(BUF_CURSOR, BUF_AVAILABLE, "%s", actionnames[A->id]);
  if (A->id == ACTION_EXEC) {
    int i = 0;
    Command_T C = A->exec;

    while (C->arg[i]) {
      snprintf(BUF_CURSOR, BUF_AVAILABLE, "%s%s", i ? " " : " '", C->arg[i]);
      i++;
    }
    snprintf(BUF_CURSOR, BUF_AVAILABLE, "'");
    if (C->has_uid)
      snprintf(BUF_CURSOR, BUF_AVAILABLE, " as uid %d", C->uid);
    if (C->has_gid)
      snprintf(BUF_CURSOR, BUF_AVAILABLE, " as gid %d", C->gid);
    snprintf(BUF_CURSOR, BUF_AVAILABLE, " timeout %d cycle(s)", C->timeout);
  }
  #undef BUF_CURSOR
  #undef BUF_AVAILABLE
}


/* Return deep copy of the command */
static Command_T copycommand(Command_T source) {
  int i;
  Command_T copy = NULL;

  NEW(copy);
  copy->length = source->length;
  copy->has_uid = source->has_uid;
  copy->uid = source->uid;
  copy->has_gid = source->has_gid;
  copy->gid = source->gid;
  copy->timeout = source->timeout;
  for (i = 0; i < copy->length; i++)
     copy->arg[i] = xstrdup(source->arg[i]);
  copy->arg[copy->length] = NULL;

  return copy;
}

#line 3509 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(void)
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = yyssp - yyss;
    newss = (yyss != 0)
          ? (short *)realloc(yyss, newsize * sizeof(*newss))
          : (short *)malloc(newsize * sizeof(*newss));
    if (newss == 0)
        return -1;

    yyss  = newss;
    yyssp = newss + i;
    newvs = (yyvs != 0)
          ? (YYSTYPE *)realloc(yyvs, newsize * sizeof(*newvs))
          : (YYSTYPE *)malloc(newsize * sizeof(*newvs));
    if (newvs == 0)
        return -1;

    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse(void)
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;

    yyerror("syntax error");

#ifdef lint
    goto yyerrlab;
#endif

yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yyvsp[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 129:
#line 474 "p.y"
{
                    addmail(yyvsp[-5].string, &mailset, &Run.maillist, eventset, yyvsp[0].number);
                  }
break;
case 130:
#line 477 "p.y"
{
                    addmail(yyvsp[-2].string, &mailset, &Run.maillist, EVENT_ALL, yyvsp[0].number);
                  }
break;
case 131:
#line 480 "p.y"
{
                   addmail(yyvsp[-6].string, &mailset, &Run.maillist, ~eventset, yyvsp[0].number);
                  }
break;
case 132:
#line 485 "p.y"
{
                    if (!Run.isdaemon || ihp.daemon) {
                      ihp.daemon     = TRUE;
                      Run.isdaemon   = TRUE;
                      Run.polltime   = yyvsp[-1].number;
                      Run.startdelay = yyvsp[0].number;
                    }
                  }
break;
case 133:
#line 495 "p.y"
{ yyval.number = START_DELAY; }
break;
case 134:
#line 496 "p.y"
{ yyval.number = yyvsp[0].number; }
break;
case 135:
#line 499 "p.y"
{
                    Run.expectbuffer = yyvsp[-1].number * yyvsp[0].number;
                  }
break;
case 136:
#line 504 "p.y"
{
                    Run.init = TRUE;
                  }
break;
case 137:
#line 509 "p.y"
{
                   if (!Run.logfile || ihp.logfile) {
                     ihp.logfile = TRUE;
                     setlogfile(yyvsp[0].string);
                     Run.use_syslog = FALSE;
                     Run.dolog =TRUE;
                   }
                  }
break;
case 138:
#line 517 "p.y"
{
                    setsyslog(NULL);
                  }
break;
case 139:
#line 520 "p.y"
{
                    setsyslog(yyvsp[0].string); FREE(yyvsp[0].string);
                  }
break;
case 140:
#line 525 "p.y"
{
                    Run.eventlist_dir = yyvsp[0].string;
                  }
break;
case 141:
#line 528 "p.y"
{
                    Run.eventlist_dir = yyvsp[-2].string;
                    Run.eventlist_slots = yyvsp[0].number;
                  }
break;
case 142:
#line 532 "p.y"
{
                    Run.eventlist_dir = xstrdup(MYEVENTLISTBASE);
                    Run.eventlist_slots = yyvsp[0].number;
                  }
break;
case 143:
#line 538 "p.y"
{
                    Run.idfile = yyvsp[0].string;
                  }
break;
case 144:
#line 543 "p.y"
{
                    Run.statefile = yyvsp[0].string;
                  }
break;
case 145:
#line 548 "p.y"
{
                   if (!Run.pidfile || ihp.pidfile) {
                     ihp.pidfile = TRUE;
                     setpidfile(yyvsp[0].string);
                   }
                 }
break;
case 149:
#line 563 "p.y"
{
                    check_hostname((yyvsp[-3].url)->hostname);
                    addmmonit(yyvsp[-3].url, yyvsp[-2].number, yyvsp[-1].number, yyvsp[0].string); 
                  }
break;
case 150:
#line 569 "p.y"
{
                   Run.mailserver_timeout = yyvsp[-1].number;
                   Run.mail_hostname = yyvsp[0].string;
                  }
break;
case 151:
#line 575 "p.y"
{
                   Run.MailFormat.from    = mailset.from    ?  mailset.from    : xstrdup(ALERT_FROM);
                   Run.MailFormat.subject = mailset.subject ?  mailset.subject : xstrdup(ALERT_SUBJECT);
                   Run.MailFormat.message = mailset.message ?  mailset.message : xstrdup(ALERT_MESSAGE);
                   reset_mailset();
                 }
break;
case 152:
#line 583 "p.y"
{
                   Run.dohttpd = TRUE;
                   Run.httpdport = yyvsp[-1].number;
                 }
break;
case 155:
#line 593 "p.y"
{
                    /* Restore the current text overriden by lookahead */
                    FREE(argyytext);
                    argyytext = xstrdup(yyvsp[-4].string);

                    check_hostname(yyvsp[-4].string);
                    mailserverset.host = yyvsp[-4].string;
                    mailserverset.username = yyvsp[-3].string;
                    mailserverset.password = yyvsp[-2].string;
                    mailserverset.ssl.version = yyvsp[-1].number;
                    if (mailserverset.ssl.version != SSL_VERSION_NONE) {
                      mailserverset.ssl.use_ssl = TRUE;
                      if (mailserverset.ssl.version == SSL_VERSION_SSLV2 ||
                         mailserverset.ssl.version == SSL_VERSION_SSLV3)
                         mailserverset.port = PORT_SMTPS;
                      mailserverset.ssl.certmd5 = yyvsp[0].string;
                    }
                    addmailserver(&mailserverset);
                  }
break;
case 156:
#line 612 "p.y"
{
                    /* Restore the current text overriden by lookahead */
                    FREE(argyytext);
                    argyytext = xstrdup(yyvsp[-6].string);

                    check_hostname(yyvsp[-6].string);
                    mailserverset.host = yyvsp[-6].string;
                    mailserverset.port = yyvsp[-4].number;
                    mailserverset.username = yyvsp[-3].string;
                    mailserverset.password = yyvsp[-2].string;
                    mailserverset.ssl.version = yyvsp[-1].number;
                    if (mailserverset.ssl.version != SSL_VERSION_NONE) {
                      mailserverset.ssl.use_ssl = TRUE;
                      mailserverset.ssl.certmd5 = yyvsp[0].string;
                    }
                    addmailserver(&mailserverset);
                  }
break;
case 163:
#line 641 "p.y"
{ Run.httpdssl = FALSE; }
break;
case 164:
#line 642 "p.y"
{ 
                    Run.httpdssl = TRUE;                   
                    if (!have_ssl())
                      yyerror("SSL is not supported");
                  }
break;
case 170:
#line 656 "p.y"
{ FREE(yyvsp[0].string); }
break;
case 171:
#line 657 "p.y"
{ FREE(yyvsp[0].string); }
break;
case 173:
#line 661 "p.y"
{ Run.httpdsig = TRUE; }
break;
case 174:
#line 662 "p.y"
{ Run.httpdsig = FALSE; }
break;
case 179:
#line 673 "p.y"
{ Run.bind_addr = yyvsp[0].string; }
break;
case 180:
#line 676 "p.y"
{
                    Run.httpsslpem = yyvsp[0].string;
                    if (!File_checkStat(Run.httpsslpem, "SSL server PEM file", S_IRWXU))
                      yyerror2("SSL server PEM file has too loose permissions");
                  }
break;
case 182:
#line 684 "p.y"
{
                    Run.httpsslclientpem = yyvsp[0].string; 
                    Run.clientssl = TRUE;
                    if (!File_checkStat(Run.httpsslclientpem, "SSL client PEM file", S_IRWXU | S_IRGRP | S_IROTH))
                      yyerror2("SSL client PEM file has too loose permissions");
                  }
break;
case 183:
#line 692 "p.y"
{ 
                   Run.allowselfcert = FALSE; 
                   config_ssl(Run.allowselfcert); 
                 }
break;
case 184:
#line 696 "p.y"
{   
                    Run.allowselfcert = TRUE;
                    config_ssl(Run.allowselfcert); 
                  }
break;
case 185:
#line 702 "p.y"
{
                    addcredentials(yyvsp[-3].string,yyvsp[-1].string, DIGEST_CLEARTEXT, yyvsp[0].number);
                  }
break;
case 186:
#line 705 "p.y"
{
#ifdef HAVE_LIBPAM
                    addpamauth(yyvsp[-1].string, yyvsp[0].number);
#else
                    yyerror("PAM is not supported");
                    FREE(yyvsp[-1].string);
#endif
                  }
break;
case 187:
#line 713 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_CLEARTEXT);
                    FREE(yyvsp[0].string);
                  }
break;
case 188:
#line 717 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_CLEARTEXT);
                    FREE(yyvsp[0].string);
                  }
break;
case 189:
#line 721 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_MD5);
                    FREE(yyvsp[0].string);
                  }
break;
case 190:
#line 725 "p.y"
{
                    addhtpasswdentry(yyvsp[0].string, NULL, DIGEST_CRYPT);
                    FREE(yyvsp[0].string);
                  }
break;
case 191:
#line 729 "p.y"
{
                    htpasswd_file = yyvsp[0].string;
                    digesttype = CLEARTEXT;
                  }
break;
case 192:
#line 733 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 193:
#line 736 "p.y"
{
                    htpasswd_file = yyvsp[0].string;
                    digesttype = DIGEST_CLEARTEXT;
                  }
break;
case 194:
#line 740 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 195:
#line 743 "p.y"
{
                    htpasswd_file = yyvsp[0].string;
                    digesttype = DIGEST_MD5;
                  }
break;
case 196:
#line 747 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 197:
#line 750 "p.y"
{
                    htpasswd_file = yyvsp[0].string;
                    digesttype = DIGEST_CRYPT;
                  }
break;
case 198:
#line 754 "p.y"
{
                    FREE(htpasswd_file);
                  }
break;
case 199:
#line 757 "p.y"
{
                    if (! (add_net_allow(yyvsp[0].string) || add_host_allow(yyvsp[0].string))) {
                      yyerror2("erroneous network or host identifier %s", yyvsp[0].string); 
                    }
                    FREE(yyvsp[0].string);
                  }
break;
case 202:
#line 769 "p.y"
{ addhtpasswdentry(htpasswd_file, yyvsp[0].string, digesttype);
                           FREE(yyvsp[0].string); }
break;
case 203:
#line 773 "p.y"
{ yyval.number = FALSE; }
break;
case 204:
#line 774 "p.y"
{ yyval.number = TRUE; }
break;
case 205:
#line 777 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_PROCESS, yyvsp[-2].string, yyvsp[0].string, check_process);
                  }
break;
case 206:
#line 781 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_PROCESS, yyvsp[-2].string, yyvsp[0].string, check_process);
                  }
break;
case 207:
#line 787 "p.y"
{
                    check_name(yyvsp[-2].string); 
                    createservice(TYPE_FILE, yyvsp[-2].string, yyvsp[0].string, check_file);
                  }
break;
case 208:
#line 793 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_FILESYSTEM, yyvsp[-2].string, yyvsp[0].string, check_filesystem);
                  }
break;
case 209:
#line 799 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_DIRECTORY, yyvsp[-2].string, yyvsp[0].string, check_directory);
                  }
break;
case 210:
#line 805 "p.y"
{
                    check_hostname(yyvsp[0].string); 
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_HOST, yyvsp[-2].string, yyvsp[0].string, check_remote_host);
                  }
break;
case 211:
#line 812 "p.y"
{
                    check_name(yyvsp[0].string);
                    createservice(TYPE_SYSTEM, yyvsp[0].string, xstrdup(""), check_system);
                    hassystem = TRUE;
                  }
break;
case 212:
#line 819 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_FIFO, yyvsp[-2].string, yyvsp[0].string, check_fifo);
                  }
break;
case 213:
#line 825 "p.y"
{
                    check_name(yyvsp[-2].string);
                    createservice(TYPE_STATUS, yyvsp[-2].string, yyvsp[0].string, check_status);
                  }
break;
case 214:
#line 831 "p.y"
{
                    addcommand(START, yyvsp[0].number);
                  }
break;
case 215:
#line 834 "p.y"
{
                    addcommand(START, yyvsp[0].number);
                  }
break;
case 216:
#line 839 "p.y"
{
                    addcommand(STOP, yyvsp[0].number);
                  }
break;
case 217:
#line 842 "p.y"
{
                    addcommand(STOP, yyvsp[0].number);
                  }
break;
case 222:
#line 855 "p.y"
{ addargument(yyvsp[0].string); }
break;
case 223:
#line 856 "p.y"
{ addargument(yyvsp[0].string); }
break;
case 224:
#line 859 "p.y"
{ addeuid( get_uid(yyvsp[0].string, 0) ); FREE(yyvsp[0].string); }
break;
case 225:
#line 860 "p.y"
{ addegid( get_gid(yyvsp[0].string, 0) ); FREE(yyvsp[0].string); }
break;
case 226:
#line 861 "p.y"
{ addeuid( get_uid(NULL, yyvsp[0].number) ); }
break;
case 227:
#line 862 "p.y"
{ addegid( get_gid(NULL, yyvsp[0].number) ); }
break;
case 228:
#line 865 "p.y"
{ yyval.string = NULL; }
break;
case 229:
#line 866 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 230:
#line 867 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 231:
#line 870 "p.y"
{ yyval.string = NULL; }
break;
case 232:
#line 871 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 233:
#line 874 "p.y"
{ yyval.string = NULL; }
break;
case 234:
#line 875 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 235:
#line 879 "p.y"
{
                    portset.timeout = yyvsp[-4].number;
                    addeventaction(&(portset).action, yyvsp[-1].number, yyvsp[0].number);
                    addport(&portset);
                  }
break;
case 236:
#line 885 "p.y"
{ 
                    prepare_urlrequest(yyvsp[-6].url);
                    portset.timeout = yyvsp[-4].number;
                    addeventaction(&(portset).action, yyvsp[-1].number, yyvsp[0].number);
                    addport(&portset);
                  }
break;
case 237:
#line 894 "p.y"
{
                   portset.timeout = yyvsp[-4].number;
                   addeventaction(&(portset).action, yyvsp[-1].number, yyvsp[0].number);
                   addport(&portset);
                  }
break;
case 238:
#line 902 "p.y"
{
                   icmpset.type = yyvsp[-6].number;
                   icmpset.count = yyvsp[-5].number;
                   icmpset.timeout = yyvsp[-4].number;
                   addeventaction(&(icmpset).action, yyvsp[-1].number, yyvsp[0].number);
                   addicmp(&icmpset);
                  }
break;
case 239:
#line 911 "p.y"
{
                    if (current->type == TYPE_HOST)
                      portset.hostname = xstrdup(current->path);
                    else
                      portset.hostname = xstrdup(LOCALHOST);
                  }
break;
case 240:
#line 917 "p.y"
{ check_hostname(yyvsp[0].string); portset.hostname = yyvsp[0].string; }
break;
case 241:
#line 920 "p.y"
{ portset.port = yyvsp[0].number; portset.family = AF_INET; }
break;
case 242:
#line 923 "p.y"
{
                    portset.pathname = yyvsp[0].string; portset.family = AF_UNIX;
                  }
break;
case 243:
#line 928 "p.y"
{
                    portset.type = SOCK_STREAM;
                  }
break;
case 244:
#line 931 "p.y"
{
                    portset.type = SOCK_STREAM;
                  }
break;
case 245:
#line 934 "p.y"
{
                    portset.type = SOCK_STREAM;
                    portset.SSL.use_ssl = TRUE;
                    portset.SSL.version = yyvsp[-1].number;
                    if (portset.SSL.version == SSL_VERSION_NONE)
                      portset.SSL.version = SSL_VERSION_AUTO;
                    portset.SSL.certmd5 = yyvsp[0].string;
                  }
break;
case 246:
#line 942 "p.y"
{
                    portset.type = SOCK_DGRAM;
                  }
break;
case 247:
#line 947 "p.y"
{ yyval.string = NULL; }
break;
case 248:
#line 948 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 249:
#line 951 "p.y"
{ yyval.number = SSL_VERSION_NONE; }
break;
case 250:
#line 952 "p.y"
{ yyval.number = SSL_VERSION_SSLV2; }
break;
case 251:
#line 953 "p.y"
{ yyval.number = SSL_VERSION_SSLV3; }
break;
case 252:
#line 954 "p.y"
{ yyval.number = SSL_VERSION_TLS; }
break;
case 253:
#line 955 "p.y"
{ yyval.number = SSL_VERSION_AUTO; }
break;
case 254:
#line 958 "p.y"
{
                    portset.protocol = addprotocol(P_DEFAULT);
                  }
break;
case 255:
#line 961 "p.y"
{
                    portset.protocol = addprotocol(P_APACHESTATUS);
                  }
break;
case 256:
#line 964 "p.y"
{
                    portset.protocol = addprotocol(P_DEFAULT);
                  }
break;
case 257:
#line 967 "p.y"
{
                    portset.protocol = addprotocol(P_DNS);
                  }
break;
case 258:
#line 970 "p.y"
{
                    portset.protocol = addprotocol(P_DWP);
                  }
break;
case 259:
#line 973 "p.y"
{
                    portset.protocol = addprotocol(P_FTP);
                  }
break;
case 260:
#line 976 "p.y"
{
                    portset.protocol = addprotocol(P_HTTP);
                  }
break;
case 261:
#line 979 "p.y"
{
                    portset.protocol = addprotocol(P_IMAP);
                  }
break;
case 262:
#line 982 "p.y"
{
                    portset.protocol = addprotocol(P_CLAMAV);
                  }
break;
case 263:
#line 985 "p.y"
{
                    portset.protocol = addprotocol(P_LDAP2);
                  }
break;
case 264:
#line 988 "p.y"
{
                    portset.protocol = addprotocol(P_LDAP3);
                  }
break;
case 265:
#line 991 "p.y"
{
                    portset.protocol = addprotocol(P_MYSQL);
                  }
break;
case 266:
#line 994 "p.y"
{
                    portset.protocol = addprotocol(P_SIP);
                  }
break;
case 267:
#line 997 "p.y"
{
                    portset.protocol = addprotocol(P_NNTP);
                  }
break;
case 268:
#line 1000 "p.y"
{
                    portset.protocol = addprotocol(P_NTP3);
                    portset.type = SOCK_DGRAM;
                  }
break;
case 269:
#line 1004 "p.y"
{
                    portset.protocol = addprotocol(P_POSTFIXPOLICY);
                  }
break;
case 270:
#line 1007 "p.y"
{
                    portset.protocol = addprotocol(P_POP);
                  }
break;
case 271:
#line 1010 "p.y"
{
                    portset.protocol = addprotocol(P_SMTP);
                  }
break;
case 272:
#line 1013 "p.y"
{
                    portset.protocol = addprotocol(P_SSH);
                  }
break;
case 273:
#line 1016 "p.y"
{
                    portset.protocol = addprotocol(P_RDATE);
                  }
break;
case 274:
#line 1019 "p.y"
{
                    portset.protocol = addprotocol(P_RSYNC);
                  }
break;
case 275:
#line 1022 "p.y"
{
                    portset.protocol = addprotocol(P_TNS);
                  }
break;
case 276:
#line 1025 "p.y"
{
                    portset.protocol = addprotocol(P_PGSQL);
                  }
break;
case 277:
#line 1028 "p.y"
{
                    portset.protocol = addprotocol(P_LMTP);
                  }
break;
case 278:
#line 1031 "p.y"
{
                    portset.protocol = addprotocol(P_GENERIC);
                  }
break;
case 281:
#line 1040 "p.y"
{ addgeneric(&portset, yyvsp[0].string, NULL); FREE(yyvsp[0].string);}
break;
case 282:
#line 1041 "p.y"
{ addgeneric(&portset, NULL, yyvsp[0].string); FREE(yyvsp[0].string);}
break;
case 284:
#line 1045 "p.y"
{
                    portset.request = yyvsp[0].string;
                  }
break;
case 285:
#line 1048 "p.y"
{
                    portset.request = yyvsp[0].string;
                  }
break;
case 287:
#line 1054 "p.y"
{
                     portset.maxforward = verifyMaxForward(yyvsp[0].number); 
                   }
break;
case 289:
#line 1060 "p.y"
{ 
                    portset.request = Util_urlEncode(yyvsp[0].string); 
                    FREE(yyvsp[0].string); 
                  }
break;
case 290:
#line 1064 "p.y"
{
                    portset.request = Util_urlEncode(yyvsp[-2].string);
                    FREE(yyvsp[-2].string);
                    portset.request_checksum = yyvsp[0].string;
                  }
break;
case 293:
#line 1075 "p.y"
{ 
                    portset.ApacheStatus.loglimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.loglimit = (int)yyvsp[-1].number; 
                  }
break;
case 294:
#line 1079 "p.y"
{ 
                    portset.ApacheStatus.closelimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.closelimit = (int)(yyvsp[-1].number); 
                  }
break;
case 295:
#line 1083 "p.y"
{ 
                    portset.ApacheStatus.dnslimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.dnslimit = (int)(yyvsp[-1].number); 
                  }
break;
case 296:
#line 1087 "p.y"
{ 
                    portset.ApacheStatus.keepalivelimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.keepalivelimit = (int)(yyvsp[-1].number); 
                  }
break;
case 297:
#line 1091 "p.y"
{ 
                    portset.ApacheStatus.replylimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.replylimit = (int)(yyvsp[-1].number); 
                  }
break;
case 298:
#line 1095 "p.y"
{ 
                    portset.ApacheStatus.requestlimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.requestlimit = (int)(yyvsp[-1].number); 
                  }
break;
case 299:
#line 1099 "p.y"
{ 
                    portset.ApacheStatus.startlimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.startlimit = (int)(yyvsp[-1].number); 
                  }
break;
case 300:
#line 1103 "p.y"
{ 
                    portset.ApacheStatus.waitlimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.waitlimit = (int)(yyvsp[-1].number); 
                  }
break;
case 301:
#line 1107 "p.y"
{ 
                    portset.ApacheStatus.gracefullimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.gracefullimit = (int)(yyvsp[-1].number); 
                  }
break;
case 302:
#line 1111 "p.y"
{ 
                    portset.ApacheStatus.cleanuplimitOP = yyvsp[-2].number; 
                    portset.ApacheStatus.cleanuplimit = (int)(yyvsp[-1].number); 
                  }
break;
case 303:
#line 1117 "p.y"
{
                    seteventaction(&(current)->action_PID, yyvsp[0].number, ACTION_IGNORE);
                  }
break;
case 304:
#line 1122 "p.y"
{
                    seteventaction(&(current)->action_PPID, yyvsp[0].number, ACTION_IGNORE);
                  }
break;
case 305:
#line 1127 "p.y"
{
                   yyval.number = ICMP_ATTEMPT_COUNT;
                  }
break;
case 306:
#line 1130 "p.y"
{    
                   yyval.number = yyvsp[0].number;
                  }
break;
case 307:
#line 1135 "p.y"
{
                   yyval.number = EXEC_TIMEOUT;
                  }
break;
case 308:
#line 1138 "p.y"
{
                   yyval.number = yyvsp[-1].number;
                  }
break;
case 309:
#line 1143 "p.y"
{
                   yyval.number = NET_TIMEOUT;
                  }
break;
case 310:
#line 1146 "p.y"
{
                   yyval.number = yyvsp[-1].number;
                  }
break;
case 311:
#line 1151 "p.y"
{
                   check_timeout(yyvsp[-5].number, yyvsp[-3].number);
                   current->def_timeout = TRUE;
                   current->to_start = yyvsp[-5].number;
                   current->to_cycle = yyvsp[-3].number;
                 }
break;
case 313:
#line 1160 "p.y"
{
                    seturlrequest(yyvsp[-1].number, yyvsp[0].string);
                    FREE(yyvsp[0].string);
                  }
break;
case 314:
#line 1166 "p.y"
{ yyval.number = OPERATOR_EQUAL; }
break;
case 315:
#line 1167 "p.y"
{ yyval.number = OPERATOR_NOTEQUAL; }
break;
case 316:
#line 1170 "p.y"
{
                   addmail(yyvsp[-5].string, &mailset, &current->maillist, eventset, yyvsp[0].number);
                  }
break;
case 317:
#line 1173 "p.y"
{
                   addmail(yyvsp[-2].string, &mailset, &current->maillist, EVENT_ALL, yyvsp[0].number);
                  }
break;
case 318:
#line 1176 "p.y"
{
                   addmail(yyvsp[-6].string, &mailset, &current->maillist, ~eventset, yyvsp[0].number);
                  }
break;
case 319:
#line 1179 "p.y"
{
                   addmail(yyvsp[0].string, &mailset, &current->maillist, EVENT_NULL, 0);
                  }
break;
case 320:
#line 1184 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 321:
#line 1187 "p.y"
{ yyval.string = yyvsp[0].string; }
break;
case 324:
#line 1194 "p.y"
{ eventset |= EVENT_ACTION; }
break;
case 325:
#line 1195 "p.y"
{ eventset |= EVENT_CHECKSUM; }
break;
case 326:
#line 1196 "p.y"
{ eventset |= EVENT_CONNECTION; }
break;
case 327:
#line 1197 "p.y"
{ eventset |= EVENT_CONTENT; }
break;
case 328:
#line 1198 "p.y"
{ eventset |= EVENT_DATA; }
break;
case 329:
#line 1199 "p.y"
{ eventset |= EVENT_EXEC; }
break;
case 330:
#line 1200 "p.y"
{ eventset |= EVENT_FSFLAG; }
break;
case 331:
#line 1201 "p.y"
{ eventset |= EVENT_GID; }
break;
case 332:
#line 1202 "p.y"
{ eventset |= EVENT_ICMP; }
break;
case 333:
#line 1203 "p.y"
{ eventset |= EVENT_INSTANCE; }
break;
case 334:
#line 1204 "p.y"
{ eventset |= EVENT_INVALID; }
break;
case 335:
#line 1205 "p.y"
{ eventset |= EVENT_NONEXIST; }
break;
case 336:
#line 1206 "p.y"
{ eventset |= EVENT_PERMISSION; }
break;
case 337:
#line 1207 "p.y"
{ eventset |= EVENT_PID; }
break;
case 338:
#line 1208 "p.y"
{ eventset |= EVENT_PPID; }
break;
case 339:
#line 1209 "p.y"
{ eventset |= EVENT_RESOURCE; }
break;
case 340:
#line 1210 "p.y"
{ eventset |= EVENT_SIZE; }
break;
case 341:
#line 1211 "p.y"
{ eventset |= EVENT_TIMEOUT; }
break;
case 342:
#line 1212 "p.y"
{ eventset |= EVENT_TIMESTAMP; }
break;
case 343:
#line 1213 "p.y"
{ eventset |= EVENT_UID; }
break;
case 348:
#line 1224 "p.y"
{ mailset.from = yyvsp[0].string; }
break;
case 349:
#line 1225 "p.y"
{ mailset.subject = yyvsp[0].string; }
break;
case 350:
#line 1226 "p.y"
{ mailset.message = yyvsp[0].string; }
break;
case 351:
#line 1229 "p.y"
{
                   check_every(yyvsp[-1].number);
                   current->def_every = TRUE;
                   current->every = yyvsp[-1].number;
                 }
break;
case 352:
#line 1236 "p.y"
{
                    current->mode = MODE_ACTIVE;
                  }
break;
case 353:
#line 1239 "p.y"
{
                    current->mode = MODE_PASSIVE;
                  }
break;
case 354:
#line 1242 "p.y"
{
                    current->mode = MODE_MANUAL;
                    current->monitor = MONITOR_NOT;
                  }
break;
case 355:
#line 1248 "p.y"
{ current->group = yyvsp[0].string; }
break;
case 359:
#line 1259 "p.y"
{ adddependant(yyvsp[0].string); }
break;
case 360:
#line 1262 "p.y"
{
                     addeventaction(&(resourceset).action, yyvsp[-1].number, yyvsp[0].number);
                     addresource(&resourceset);
                   }
break;
case 367:
#line 1278 "p.y"
{
                     addeventaction(&(resourceset).action, yyvsp[-1].number, yyvsp[0].number);
                     addresource(&resourceset);
                   }
break;
case 373:
#line 1293 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_CPU_PERCENT;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10); 
                  }
break;
case 374:
#line 1298 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_TOTAL_CPU_PERCENT;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10); 
                  }
break;
case 375:
#line 1305 "p.y"
{
                    resourceset.resource_id = yyvsp[-3].number;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10); 
                  }
break;
case 376:
#line 1312 "p.y"
{ yyval.number = RESOURCE_ID_CPUUSER; }
break;
case 377:
#line 1313 "p.y"
{ yyval.number = RESOURCE_ID_CPUSYSTEM; }
break;
case 378:
#line 1314 "p.y"
{ yyval.number = RESOURCE_ID_CPUWAIT; }
break;
case 379:
#line 1317 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_MEM_KBYTE;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (int) (yyvsp[-1].real * (yyvsp[0].number / 1024.0)); 
                  }
break;
case 380:
#line 1322 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_MEM_PERCENT;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10); 
                  }
break;
case 381:
#line 1327 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_TOTAL_MEM_KBYTE;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (int) (yyvsp[-1].real * (yyvsp[0].number / 1024.0));
                  }
break;
case 382:
#line 1332 "p.y"
{
                    resourceset.resource_id = RESOURCE_ID_TOTAL_MEM_PERCENT;
                    resourceset.operator = yyvsp[-2].number;
                    resourceset.limit = (yyvsp[-1].number * 10);
                  }
break;
case 383:
#line 1339 "p.y"
{ 
                    resourceset.resource_id = RESOURCE_ID_CHILDREN;
                    resourceset.operator = yyvsp[-1].number;
                    resourceset.limit = (int) yyvsp[0].number; 
                  }
break;
case 384:
#line 1346 "p.y"
{ 
                    resourceset.resource_id = yyvsp[-2].number;
                    resourceset.operator = yyvsp[-1].number;
                    resourceset.limit = (int) (yyvsp[0].real * 10.0); 
                  }
break;
case 385:
#line 1353 "p.y"
{ yyval.number = RESOURCE_ID_LOAD1; }
break;
case 386:
#line 1354 "p.y"
{ yyval.number = RESOURCE_ID_LOAD5; }
break;
case 387:
#line 1355 "p.y"
{ yyval.number = RESOURCE_ID_LOAD15; }
break;
case 388:
#line 1358 "p.y"
{ yyval.real = yyvsp[0].real; }
break;
case 389:
#line 1359 "p.y"
{ yyval.real = (float) yyvsp[0].number; }
break;
case 390:
#line 1363 "p.y"
{
                    timestampset.operator = yyvsp[-6].number;
                    timestampset.time = (yyvsp[-5].number * yyvsp[-4].number);
                    addeventaction(&(timestampset).action, yyvsp[-1].number, yyvsp[0].number);
                    addtimestamp(&timestampset, FALSE);
                  }
break;
case 391:
#line 1369 "p.y"
{
                    timestampset.test_changes = TRUE;
                    addeventaction(&(timestampset).action, yyvsp[0].number, ACTION_IGNORE);
                    addtimestamp(&timestampset, TRUE);
                  }
break;
case 392:
#line 1376 "p.y"
{ yyval.number = OPERATOR_EQUAL; }
break;
case 393:
#line 1377 "p.y"
{ yyval.number = OPERATOR_GREATER; }
break;
case 394:
#line 1378 "p.y"
{ yyval.number = OPERATOR_LESS; }
break;
case 395:
#line 1379 "p.y"
{ yyval.number = OPERATOR_EQUAL; }
break;
case 396:
#line 1380 "p.y"
{ yyval.number = OPERATOR_NOTEQUAL; }
break;
case 397:
#line 1381 "p.y"
{ yyval.number = OPERATOR_NOTEQUAL; }
break;
case 398:
#line 1384 "p.y"
{ yyval.number = TIME_SECOND; }
break;
case 399:
#line 1385 "p.y"
{ yyval.number = TIME_SECOND; }
break;
case 400:
#line 1386 "p.y"
{ yyval.number = TIME_MINUTE; }
break;
case 401:
#line 1387 "p.y"
{ yyval.number = TIME_HOUR; }
break;
case 402:
#line 1388 "p.y"
{ yyval.number = TIME_DAY; }
break;
case 403:
#line 1391 "p.y"
{ yyval.number = ACTION_ALERT; }
break;
case 404:
#line 1392 "p.y"
{ yyval.number = ACTION_EXEC; }
break;
case 405:
#line 1393 "p.y"
{ yyval.number = ACTION_EXEC; }
break;
case 406:
#line 1394 "p.y"
{ yyval.number = ACTION_RESTART; }
break;
case 407:
#line 1395 "p.y"
{ yyval.number = ACTION_START; }
break;
case 408:
#line 1396 "p.y"
{ yyval.number = ACTION_STOP; }
break;
case 409:
#line 1397 "p.y"
{ yyval.number = ACTION_UNMONITOR; }
break;
case 410:
#line 1400 "p.y"
{
                    yyval.number = yyvsp[0].number;
                    if (yyvsp[0].number == ACTION_EXEC && command) {
                      command1 = command;
                      command = NULL;
                    }
                  }
break;
case 411:
#line 1409 "p.y"
{
                    yyval.number = yyvsp[0].number;
                    if (yyvsp[0].number == ACTION_EXEC && command) {
                      command2 = command;
                      command = NULL;
                    }
                  }
break;
case 413:
#line 1419 "p.y"
{
                    rate1.count  = yyvsp[-1].number;
                    rate1.cycles = yyvsp[-1].number;
                    if (rate1.cycles < 1 || rate1.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                  }
break;
case 414:
#line 1425 "p.y"
{
                    rate1.count  = yyvsp[-2].number;
                    rate1.cycles = yyvsp[-1].number;
                    if (rate1.cycles < 1 || rate1.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                    if (rate1.count < 1 || rate1.count > rate1.cycles)
                      yyerror2("the number of events must be bigger then 0 and less than poll cycles");
                  }
break;
case 416:
#line 1436 "p.y"
{
                    rate2.count  = yyvsp[-1].number;
                    rate2.cycles = yyvsp[-1].number;
                    if (rate2.cycles < 1 || rate2.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                  }
break;
case 417:
#line 1442 "p.y"
{
                    rate2.count  = yyvsp[-2].number;
                    rate2.cycles = yyvsp[-1].number;
                    if (rate2.cycles < 1 || rate2.cycles > BITMAP_MAX)
                      yyerror2("the number of cycles must be between 1 and %d", BITMAP_MAX);
                    if (rate2.count < 1 || rate2.count > rate2.cycles)
                      yyerror2("the number of events must be bigger then 0 and less than poll cycles");
                  }
break;
case 418:
#line 1452 "p.y"
{
                    yyval.number = ACTION_ALERT;
                  }
break;
case 419:
#line 1455 "p.y"
{
                    yyval.number = yyvsp[0].number;
                  }
break;
case 420:
#line 1458 "p.y"
{
                    yyval.number = yyvsp[0].number;
                  }
break;
case 421:
#line 1461 "p.y"
{
                    yyval.number = yyvsp[0].number;
                  }
break;
case 422:
#line 1466 "p.y"
{
                    addeventaction(&(checksumset).action, yyvsp[-1].number, yyvsp[0].number);
                    addchecksum(&checksumset);
                  }
break;
case 423:
#line 1471 "p.y"
{
                    checksumset.hash = yyvsp[-4].string;
                    addeventaction(&(checksumset).action, yyvsp[-1].number, yyvsp[0].number);
                    addchecksum(&checksumset);
                  }
break;
case 424:
#line 1476 "p.y"
{
                    checksumset.test_changes = TRUE;
                    addeventaction(&(checksumset).action, yyvsp[0].number, ACTION_IGNORE);
                    addchecksum(&checksumset);
                  }
break;
case 425:
#line 1482 "p.y"
{ checksumset.type = HASH_UNKNOWN; }
break;
case 426:
#line 1483 "p.y"
{ checksumset.type = HASH_MD5; }
break;
case 427:
#line 1484 "p.y"
{ checksumset.type = HASH_SHA1; }
break;
case 428:
#line 1487 "p.y"
{
                    filesystemset.resource = RESOURCE_ID_INODE;
                    filesystemset.operator = yyvsp[-5].number;
                    filesystemset.limit_absolute = yyvsp[-4].number;
                    addeventaction(&(filesystemset).action, yyvsp[-1].number, yyvsp[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 429:
#line 1494 "p.y"
{
                    filesystemset.resource = RESOURCE_ID_INODE;
                    filesystemset.operator = yyvsp[-6].number;
                    filesystemset.limit_percent = (int)(yyvsp[-5].number * 10);
                    addeventaction(&(filesystemset).action, yyvsp[-1].number, yyvsp[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 430:
#line 1503 "p.y"
{
                    if (!filesystem_usage(current->inf, current->path))
                      yyerror2("cannot read usage of filesystem %s", current->path);
                    filesystemset.resource = RESOURCE_ID_SPACE;
                    filesystemset.operator = yyvsp[-6].number;
                    filesystemset.limit_absolute = (int)((float)yyvsp[-5].real / (float)current->inf->f_bsize * (float)yyvsp[-4].number);
                    addeventaction(&(filesystemset).action, yyvsp[-1].number, yyvsp[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 431:
#line 1512 "p.y"
{
                    filesystemset.resource = RESOURCE_ID_SPACE;
                    filesystemset.operator = yyvsp[-6].number;
                    filesystemset.limit_percent = (int)(yyvsp[-5].number * 10);
                    addeventaction(&(filesystemset).action, yyvsp[-1].number, yyvsp[0].number);
                    addfilesystem(&filesystemset);
                  }
break;
case 432:
#line 1521 "p.y"
{
                    seteventaction(&(current)->action_FSFLAG, yyvsp[0].number, ACTION_IGNORE);
                  }
break;
case 433:
#line 1526 "p.y"
{ yyval.number = UNIT_BYTE; }
break;
case 434:
#line 1527 "p.y"
{ yyval.number = UNIT_BYTE; }
break;
case 435:
#line 1528 "p.y"
{ yyval.number = UNIT_KILOBYTE; }
break;
case 436:
#line 1529 "p.y"
{ yyval.number = UNIT_MEGABYTE; }
break;
case 437:
#line 1530 "p.y"
{ yyval.number = UNIT_GIGABYTE; }
break;
case 438:
#line 1533 "p.y"
{
                    permset.perm = check_perm(yyvsp[-4].number);
                    addeventaction(&(permset).action, yyvsp[-1].number, yyvsp[0].number);
                    addperm(&permset);
                  }
break;
case 439:
#line 1540 "p.y"
{
                    matchset.ignore = FALSE;
                    matchset.match_path = yyvsp[-3].string;
                    matchset.match_string = NULL;
                    addmatchpath(&matchset, yyvsp[0].number);
                    FREE(yyvsp[-3].string); 
                  }
break;
case 440:
#line 1547 "p.y"
{
                    matchset.ignore = FALSE;
                    matchset.match_path = NULL;
                    matchset.match_string = xstrdup(yyvsp[-3].string);
                    addmatch(&matchset, yyvsp[0].number, 0);
                  }
break;
case 441:
#line 1553 "p.y"
{
                    matchset.ignore = TRUE;
                    matchset.match_path = yyvsp[0].string;
                    matchset.match_string = NULL;
                    addmatchpath(&matchset, ACTION_IGNORE);
                    FREE(yyvsp[0].string); 
                  }
break;
case 442:
#line 1560 "p.y"
{
                    matchset.ignore = TRUE;
                    matchset.match_path = NULL;
                    matchset.match_string = xstrdup(yyvsp[0].string);
                    addmatch(&matchset, ACTION_IGNORE, 0);
                  }
break;
case 443:
#line 1568 "p.y"
{
                    matchset.not = FALSE;
                  }
break;
case 444:
#line 1571 "p.y"
{
                    matchset.not = TRUE;
                  }
break;
case 445:
#line 1577 "p.y"
{
                    sizeset.operator = yyvsp[-6].number;
                    sizeset.size = ((unsigned long long)yyvsp[-5].number * yyvsp[-4].number);
                    addeventaction(&(sizeset).action, yyvsp[-1].number, yyvsp[0].number);
                    addsize(&sizeset);
                  }
break;
case 446:
#line 1583 "p.y"
{
                    sizeset.test_changes = TRUE;
                    addeventaction(&(sizeset).action, yyvsp[0].number, ACTION_IGNORE);
                    addsize(&sizeset);
                  }
break;
case 447:
#line 1590 "p.y"
{
                    uidset.uid = get_uid(yyvsp[-4].string, 0);
                    addeventaction(&(uidset).action, yyvsp[-1].number, yyvsp[0].number);
                    adduid(&uidset);
                    FREE(yyvsp[-4].string);
                  }
break;
case 448:
#line 1596 "p.y"
{
                    uidset.uid = get_uid(NULL, yyvsp[-4].number);
                    addeventaction(&(uidset).action, yyvsp[-1].number, yyvsp[0].number);
                    adduid(&uidset);
                  }
break;
case 449:
#line 1603 "p.y"
{
                    gidset.gid = get_gid(yyvsp[-4].string, 0);
                    addeventaction(&(gidset).action, yyvsp[-1].number, yyvsp[0].number);
                    addgid(&gidset);
                    FREE(yyvsp[-4].string);
                  }
break;
case 450:
#line 1609 "p.y"
{
                    gidset.gid = get_gid(NULL, yyvsp[-4].number);
                    addeventaction(&(gidset).action, yyvsp[-1].number, yyvsp[0].number);
                    addgid(&gidset);
                  }
break;
case 451:
#line 1616 "p.y"
{ yyval.number = ICMP_ECHO; }
break;
case 452:
#line 1619 "p.y"
{ yyval.number = 0; }
break;
case 453:
#line 1620 "p.y"
{ yyval.number = yyvsp[0].number; }
break;
case 454:
#line 1621 "p.y"
{ yyval.number = yyvsp[-1].number; }
break;
#line 5325 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;

yyoverflow:
    yyerror("yacc stack overflow");

yyabort:
    return (1);

yyaccept:
    return (0);
}
